# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Hadesd(KaitaiStruct):
    """# General
    :field hadesd_type: cubesat_frame.first_header.type
    :field hadesd_address: cubesat_frame.first_header.address
    # Type1 Frame (Power):
    :field hadesd_power_spa_tmp: cubesat_frame.metadata.spa_tmp
    :field hadesd_power_spb_tmp: cubesat_frame.metadata.spb_tmp
    :field hadesd_power_spc_tmp: cubesat_frame.metadata.spc_tmp
    :field hadesd_power_spd_tmp: cubesat_frame.metadata.spd_tmp
    :field hadesd_power_spe_tmp: cubesat_frame.metadata.spe_tmp
    :field hadesd_power_spf_tmp: cubesat_frame.metadata.spf_tmp
    :field hadesd_power_vbusadc_vbatadc_hi_tmp: cubesat_frame.metadata.vbusadc_vbatadc_hi_tmp
    :field hadesd_power_vbatadc_lo_vcpuadc_hi_tmp: cubesat_frame.metadata.vbatadc_lo_vcpuadc_hi_tmp
    :field hadesd_power_vcpuadc_lo_vbus2_tmp: cubesat_frame.metadata.vcpuadc_lo_vbus2_tmp
    :field hadesd_power_vbus3_vbat2_hi_tmp: cubesat_frame.metadata.vbus3_vbat2_hi_tmp
    :field hadesd_power_vbat2_lo_ibat_hi_tmp: cubesat_frame.metadata.vbat2_lo_ibat_hi_tmp
    :field hadesd_power_ibat_lo_icpu_hi_tmp: cubesat_frame.metadata.ibat_lo_icpu_hi_tmp
    :field hadesd_power_icpu_lo_ipl_tmp: cubesat_frame.metadata.icpu_lo_ipl_tmp
    :field hadesd_power_powerdul_tmp: cubesat_frame.metadata.powerdul_tmp
    :field hadesd_power_powerdul455_tmp: cubesat_frame.metadata.powerdul455_tmp
    :field hadesd_power_vdac_tmp: cubesat_frame.metadata.vdac_tmp
    :field hadesd_power_spa_dec: cubesat_frame.metadata.spa_dec
    :field hadesd_power_spb_dec: cubesat_frame.metadata.spb_dec
    :field hadesd_power_spc_dec: cubesat_frame.metadata.spc_dec
    :field hadesd_power_spd_dec: cubesat_frame.metadata.spd_dec
    :field hadesd_power_vbus1_dec: cubesat_frame.metadata.vbus1_dec
    :field hadesd_power_vbus2_dec: cubesat_frame.metadata.vbus2_dec
    :field hadesd_power_vbus3_dec: cubesat_frame.metadata.vbus3_dec
    :field hadesd_power_vbat1_dec: cubesat_frame.metadata.vbat1_dec
    :field hadesd_power_vbat2_dec: cubesat_frame.metadata.vbat2_dec
    :field hadesd_power_vbus1_vbat1_dec: cubesat_frame.metadata.vbus1_vbat1_dec
    :field hadesd_power_vbus3_vbus2_dec: cubesat_frame.metadata.vbus3_vbus2_dec
    :field hadesd_power_vcpu_dec: cubesat_frame.metadata.vcpu_dec
    :field hadesd_power_icpu_dec: cubesat_frame.metadata.icpu_dec
    :field hadesd_power_ipl_dec: cubesat_frame.metadata.ipl_dec 
    :field hadesd_power_ibat_dec: cubesat_frame.metadata.ibat_dec
    :field hadesd_power_pwrdul1_dec: cubesat_frame.metadata.pwrdul1_dec
    :field hadesd_power_pwrdul4_dec: cubesat_frame.metadata.pwrdul4_dec
    # Type2 Frame (Temp):
    :field hadesd_temp_tpa_tmp: cubesat_frame.metadata.tpa_tmp
    :field hadesd_temp_tpb_tmp: cubesat_frame.metadata.tpb_tmp
    :field hadesd_temp_tpc_tmp: cubesat_frame.metadata.tpc_tmp
    :field hadesd_temp_tpd_tmp: cubesat_frame.metadata.tpd_tmp
    :field hadesd_temp_tpe_tmp: cubesat_frame.metadata.tpe_tmp
    :field hadesd_temp_teps_tmp: cubesat_frame.metadata.teps_tmp
    :field hadesd_temp_ttx_tmp: cubesat_frame.metadata.ttx_tmp
    :field hadesd_temp_ttx2_tmp: cubesat_frame.metadata.ttx2_tmp
    :field hadesd_temp_trx_tmp: cubesat_frame.metadata.trx_tmp
    :field hadesd_temp_tcpu_tmp: cubesat_frame.metadata.tcpu_tmp
    :field hadesd_temp_tpa_dec: cubesat_frame.metadata.tpa_dec
    :field hadesd_temp_tpb_dec: cubesat_frame.metadata.tpb_dec
    :field hadesd_temp_tpc_dec: cubesat_frame.metadata.tpc_dec
    :field hadesd_temp_tpd_dec: cubesat_frame.metadata.tpd_dec
    :field hadesd_temp_teps_dec: cubesat_frame.metadata.teps_dec
    :field hadesd_temp_ttx_dec: cubesat_frame.metadata.ttx_dec
    :field hadesd_temp_ttx2_dec: cubesat_frame.metadata.ttx2_dec
    :field hadesd_temp_trx_dec: cubesat_frame.metadata.trx_dec
    :field hadesd_temp_tcpu_dec: cubesat_frame.metadata.tcpu_dec
    # Type3 Frame (Status):
    :field hadesd_status_sclock_dec: cubesat_frame.metadata.sclock_dec
    :field hadesd_status_uptime_dec: cubesat_frame.metadata.uptime_dec
    :field hadesd_status_nrun_dec: cubesat_frame.metadata.nrun_dec
    :field hadesd_status_npayload_dec: cubesat_frame.metadata.npayload_dec
    :field hadesd_status_nwire_dec: cubesat_frame.metadata.nwire_dec
    :field hadesd_status_nbusdrops_lastreset_tmp: cubesat_frame.metadata.nbusdrops_lastreset_tmp
    :field hadesd_status_bate_mote_tmp: cubesat_frame.metadata.bate_mote_tmp
    :field hadesd_status_ntasksnotexecuted_dec: cubesat_frame.metadata.ntasksnotexecuted_dec
    :field hadesd_status_antennadeployed_dec: cubesat_frame.metadata.antennadeployed_dec
    :field hadesd_status_nexteepromerrors_dec: cubesat_frame.metadata.nexteepromerrors_dec
    :field hadesd_status_failed_task_id_dec: cubesat_frame.metadata.failed_task_id_dec
    :field hadesd_status_orbperiod_dec: cubesat_frame.metadata.orbperiod_dec
    :field hadesd_status_strfwd0_dec: cubesat_frame.metadata.strfwd0_dec
    :field hadesd_status_strfwd1_dec: cubesat_frame.metadata.strfwd1_dec
    :field hadesd_status_strfwd2_dec: cubesat_frame.metadata.strfwd2_dec
    :field hadesd_status_strfwd3_dec: cubesat_frame.metadata.strfwd3_dec
    :field hadesd_status_nbusdrops_dec: cubesat_frame.metadata.nbusdrops_dec
    :field hadesd_status_ntrans_dec: cubesat_frame.metadata.ntrans_dec
    :field hadesd_status_last_reset_cause_dec: cubesat_frame.metadata.last_reset_cause_dec
    :field hadesd_status_battery_dec: cubesat_frame.metadata.battery_dec
    :field hadesd_status_transponder_dec: cubesat_frame.metadata.transponder_dec
    # Type4 Frame (PowerStats):
    :field hadesd_powerstats_minvbusadc_vbatadc_hi_tmp: cubesat_frame.metadata.minvbusadc_vbatadc_hi_tmp
    :field hadesd_powerstats_minvbatadc_lo_vcpuadc_hi_tmp: cubesat_frame.metadata.minvbatadc_lo_vcpuadc_hi_tmp
    :field hadesd_powerstats_minvcpuadc_lo_free_tmp: cubesat_frame.metadata.minvcpuadc_lo_free_tmp
    :field hadesd_powerstats_minvbus2_tmp: cubesat_frame.metadata.minvbus2_tmp
    :field hadesd_powerstats_minvbus3_tmp: cubesat_frame.metadata.minvbus3_tmp
    :field hadesd_powerstats_minvbat2_tmp: cubesat_frame.metadata.minvbat2_tmp
    :field hadesd_powerstats_minibat_tmp: cubesat_frame.metadata.minibat_tmp
    :field hadesd_powerstats_minicpu_dec: cubesat_frame.metadata.minicpu_dec
    :field hadesd_powerstats_minipl_dec: cubesat_frame.metadata.minipl_dec
    :field hadesd_powerstats_minpowerdul_dec: cubesat_frame.metadata.minpowerdul_dec
    :field hadesd_powerstats_minpowerdul455_dec: cubesat_frame.metadata.minpowerdul455_dec
    :field hadesd_powerstats_minvdac_dec: cubesat_frame.metadata.minvdac_dec
    :field hadesd_powerstats_maxvbusadc_vbatadc_hi_tmp: cubesat_frame.metadata.maxvbusadc_vbatadc_hi_tmp
    :field hadesd_powerstats_maxvbatadc_lo_vcpuadc_hi_tmp: cubesat_frame.metadata.maxvbatadc_lo_vcpuadc_hi_tmp
    :field hadesd_powerstats_maxvcpuadc_lo_free_tmp: cubesat_frame.metadata.maxvcpuadc_lo_free_tmp
    :field hadesd_powerstats_maxvbus2_tmp: cubesat_frame.metadata.maxvbus2_tmp
    :field hadesd_powerstats_maxvbus3_tmp: cubesat_frame.metadata.maxvbus3_tmp
    :field hadesd_powerstats_maxvbat2_tmp: cubesat_frame.metadata.maxvbat2_tmp
    :field hadesd_powerstats_maxibat_dec: cubesat_frame.metadata.maxibat_dec
    :field hadesd_powerstats_maxicpu_dec: cubesat_frame.metadata.maxicpu_dec
    :field hadesd_powerstats_maxipl_tmp: cubesat_frame.metadata.maxipl_tmp
    :field hadesd_powerstats_maxpowerdul_dec: cubesat_frame.metadata.maxpowerdul_dec
    :field hadesd_powerstats_maxpowerdul455_dec: cubesat_frame.metadata.maxpowerdul455_dec
    :field hadesd_powerstats_maxvdac_dec: cubesat_frame.metadata.maxvdac_dec
    :field hadesd_powerstats_medvbusadc_vbatadc_hi_tmp: cubesat_frame.metadata.medvbusadc_vbatadc_hi_tmp
    :field hadesd_powerstats_medvbatadc_lo_vcpuadc_hi_tmp: cubesat_frame.metadata.medvbatadc_lo_vcpuadc_hi_tmp
    :field hadesd_powerstats_medvcpuadc_lo_free_tmp: cubesat_frame.metadata.medvcpuadc_lo_free_tmp
    :field hadesd_powerstats_ibat_rx_charging_dec: cubesat_frame.metadata.ibat_rx_charging_dec
    :field hadesd_powerstats_ibat_rx_discharging_dec: cubesat_frame.metadata.ibat_rx_discharging_dec
    :field hadesd_powerstats_ibat_tx_low_power_charging_dec: cubesat_frame.metadata.ibat_tx_low_power_charging_dec
    :field hadesd_powerstats_ibat_tx_low_power_discharging_dec: cubesat_frame.metadata.ibat_tx_low_power_discharging_dec
    :field hadesd_powerstats_ibat_tx_high_power_charging_dec: cubesat_frame.metadata.ibat_tx_high_power_charging_dec
    :field hadesd_powerstats_ibat_tx_high_power_discharging_dec: cubesat_frame.metadata.ibat_tx_high_power_discharging_dec
    :field hadesd_powerstats_medpowerdul_dec: cubesat_frame.metadata.medpowerdul_dec
    :field hadesd_powerstats_medpowerdul455_dec: cubesat_frame.metadata.medpowerdul455_dec
    :field hadesd_powerstats_medvdac_dec: cubesat_frame.metadata.medvdac_dec
    :field hadesd_powerstats_minvbus1_dec: cubesat_frame.metadata.minvbus1_dec
    :field hadesd_powerstats_minvbat1_dec: cubesat_frame.metadata.minvbat1_dec
    :field hadesd_powerstats_minvcpu_dec: cubesat_frame.metadata.minvcpu_dec
    :field hadesd_powerstats_minvbus2_dec: cubesat_frame.metadata.minvbus2_dec
    :field hadesd_powerstats_minvbus3_dec: cubesat_frame.metadata.minvbus3_dec
    :field hadesd_powerstats_minvbat2_dec: cubesat_frame.metadata.minvbat2_dec
    :field hadesd_powerstats_minibat_dec: cubesat_frame.metadata.minibat_dec
    :field hadesd_powerstats_maxvbus1_dec: cubesat_frame.metadata.maxvbus1_dec
    :field hadesd_powerstats_maxvbat1_dec: cubesat_frame.metadata.maxvbat1_dec
    :field hadesd_powerstats_maxvcpu_dec: cubesat_frame.metadata.maxvcpu_dec
    :field hadesd_powerstats_maxvbus2_dec: cubesat_frame.metadata.maxvbus2_dec
    :field hadesd_powerstats_maxvbus3_dec: cubesat_frame.metadata.maxvbus3_dec
    :field hadesd_powerstats_maxvbat2_dec: cubesat_frame.metadata.maxvbat2_dec
    :field hadesd_powerstats_maxipl_dec: cubesat_frame.metadata.maxipl_dec
    # Type 5 Frame (TempStats):
    :field hadesd_tempstats_mintpa_tmp: cubesat_frame.metadata.mintpa_tmp
    :field hadesd_tempstats_mintpb_tmp: cubesat_frame.metadata.mintpb_tmp
    :field hadesd_tempstats_mintpc_tmp: cubesat_frame.metadata.mintpc_tmp
    :field hadesd_tempstats_mintpd_tmp: cubesat_frame.metadata.mintpd_tmp
    :field hadesd_tempstats_mintpe_tmp: cubesat_frame.metadata.mintpe_tmp
    :field hadesd_tempstats_minteps_tmp: cubesat_frame.metadata.minteps_tmp
    :field hadesd_tempstats_minttx_tmp: cubesat_frame.metadata.minttx_tmp
    :field hadesd_tempstats_minttx2_tmp: cubesat_frame.metadata.minttx2_tmp
    :field hadesd_tempstats_mintrx_tmp: cubesat_frame.metadata.mintrx_tmp
    :field hadesd_tempstats_mintcpu_tmp: cubesat_frame.metadata.mintcpu_tmp
    :field hadesd_tempstats_maxtpa_tmp: cubesat_frame.metadata.maxtpa_tmp
    :field hadesd_tempstats_maxtpb_tmp: cubesat_frame.metadata.maxtpb_tmp
    :field hadesd_tempstats_maxtpc_tmp: cubesat_frame.metadata.maxtpc_tmp
    :field hadesd_tempstats_maxtpd_tmp: cubesat_frame.metadata.maxtpd_tmp
    :field hadesd_tempstats_maxtpe_tmp: cubesat_frame.metadata.maxtpe_tmp
    :field hadesd_tempstats_maxteps_tmp: cubesat_frame.metadata.maxteps_tmp
    :field hadesd_tempstats_maxttx_tmp: cubesat_frame.metadata.maxttx_tmp
    :field hadesd_tempstats_maxttx2_tmp: cubesat_frame.metadata.maxttx2_tmp
    :field hadesd_tempstats_maxtrx_tmp: cubesat_frame.metadata.maxtrx_tmp
    :field hadesd_tempstats_maxtcpu_tmp: cubesat_frame.metadata.maxtcpu_tmp
    :field hadesd_tempstats_medtpa_tmp: cubesat_frame.metadata.medtpa_tmp
    :field hadesd_tempstats_medtpb_tmp: cubesat_frame.metadata.medtpb_tmp
    :field hadesd_tempstats_medtpc_tmp: cubesat_frame.metadata.medtpc_tmp
    :field hadesd_tempstats_medtpd_tmp: cubesat_frame.metadata.medtpd_tmp
    :field hadesd_tempstats_medtpe_tmp: cubesat_frame.metadata.medtpe_tmp
    :field hadesd_tempstats_medteps_tmp: cubesat_frame.metadata.medteps_tmp
    :field hadesd_tempstats_medttx_tmp: cubesat_frame.metadata.medttx_tmp
    :field hadesd_tempstats_medttx2_tmp: cubesat_frame.metadata.medttx2_tmp
    :field hadesd_tempstats_medtrx_tmp: cubesat_frame.metadata.medtrx_tmp
    :field hadesd_tempstats_medtcpu_tmp: cubesat_frame.metadata.medtcpu_tmp
    :field hadesd_tempstats_mintpa_dec: cubesat_frame.metadata.mintpa_dec
    :field hadesd_tempstats_mintpb_dec: cubesat_frame.metadata.mintpb_dec
    :field hadesd_tempstats_mintpc_dec: cubesat_frame.metadata.mintpc_dec
    :field hadesd_tempstats_mintpd_dec: cubesat_frame.metadata.mintpd_dec
    :field hadesd_tempstats_minteps_dec: cubesat_frame.metadata.minteps_dec
    :field hadesd_tempstats_minttx_dec: cubesat_frame.metadata.minttx_dec
    :field hadesd_tempstats_minttx2_dec: cubesat_frame.metadata.minttx2_dec
    :field hadesd_tempstats_mintrx_dec: cubesat_frame.metadata.mintrx_dec
    :field hadesd_tempstats_mintcpu_dec: cubesat_frame.metadata.mintcpu_dec
    :field hadesd_tempstats_maxtpa_dec: cubesat_frame.metadata.maxtpa_dec
    :field hadesd_tempstats_maxtpb_dec: cubesat_frame.metadata.maxtpb_dec
    :field hadesd_tempstats_maxtpc_dec: cubesat_frame.metadata.maxtpc_dec
    :field hadesd_tempstats_maxtpd_dec: cubesat_frame.metadata.maxtpd_dec
    :field hadesd_tempstats_maxteps_dec: cubesat_frame.metadata.maxteps_dec
    :field hadesd_tempstats_maxttx_dec: cubesat_frame.metadata.maxttx_dec
    :field hadesd_tempstats_maxttx2_dec: cubesat_frame.metadata.maxttx2_dec
    :field hadesd_tempstats_maxtrx_dec: cubesat_frame.metadata.maxtrx_dec
    :field hadesd_tempstats_maxtcpu_dec: cubesat_frame.metadata.maxtcpu_dec
    :field hadesd_tempstats_medtpa_dec: cubesat_frame.metadata.medtpa_dec
    :field hadesd_tempstats_medtpb_dec: cubesat_frame.metadata.medtpb_dec
    :field hadesd_tempstats_medtpc_dec: cubesat_frame.metadata.medtpc_dec
    :field hadesd_tempstats_medtpd_dec: cubesat_frame.metadata.medtpd_dec
    :field hadesd_tempstats_medteps_dec: cubesat_frame.metadata.medteps_dec
    :field hadesd_tempstats_medttx_dec: cubesat_frame.metadata.medttx_dec
    :field hadesd_tempstats_medttx2_dec: cubesat_frame.metadata.medttx2_dec
    :field hadesd_tempstats_medtrx_dec: cubesat_frame.metadata.medtrx_dec
    :field hadesd_tempstats_medtcpu_dec: cubesat_frame.metadata.medtcpu_dec
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.cubesat_frame = Hadesd.CubesatFrame(self._io, self, self._root)

    class PowerFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.spa_tmp = self._io.read_u1()
            self.spb_tmp = self._io.read_u1()
            self.spc_tmp = self._io.read_u1()
            self.spd_tmp = self._io.read_u1()
            self.spe_tmp = self._io.read_u1()
            self.spf_tmp = self._io.read_u1()
            self.vbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.vbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.vcpuadc_lo_vbus2_tmp = self._io.read_u2le()
            self.vbus3_vbat2_hi_tmp = self._io.read_u2le()
            self.vbat2_lo_ibat_hi_tmp = self._io.read_u2le()
            self.ibat_lo_icpu_hi_tmp = self._io.read_u2le()
            self.icpu_lo_ipl_tmp = self._io.read_u2le()
            self.powerdul_tmp = self._io.read_u1()
            self.powerdul455_tmp = self._io.read_u1()
            self.vdac_tmp = self._io.read_u1()

        @property
        def pwrdul1_dec(self):
            if hasattr(self, '_m_pwrdul1_dec'):
                return self._m_pwrdul1_dec

            self._m_pwrdul1_dec = (self.powerdul_tmp * 16)
            return getattr(self, '_m_pwrdul1_dec', None)

        @property
        def vbus3_vbus2_dec(self):
            if hasattr(self, '_m_vbus3_vbus2_dec'):
                return self._m_vbus3_vbus2_dec

            self._m_vbus3_vbus2_dec = (self.vbus3_dec - self.vbus2_dec)
            return getattr(self, '_m_vbus3_vbus2_dec', None)

        @property
        def spc_dec(self):
            if hasattr(self, '_m_spc_dec'):
                return self._m_spc_dec

            self._m_spc_dec = (self.spc_tmp << 1)
            return getattr(self, '_m_spc_dec', None)

        @property
        def ibat_dec(self):
            if hasattr(self, '_m_ibat_dec'):
                return self._m_ibat_dec

            self._m_ibat_dec = (((((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)) - 4096) if (((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)) > 2047 else (((self.vbat2_lo_ibat_hi_tmp << 8) & 3840) | ((self.ibat_lo_icpu_hi_tmp >> 8) & 4095)))
            return getattr(self, '_m_ibat_dec', None)

        @property
        def vbat1_dec(self):
            if hasattr(self, '_m_vbat1_dec'):
                return self._m_vbat1_dec

            self._m_vbat1_dec = ((((self.vbatadc_lo_vcpuadc_hi_tmp << 8) & 3840) | ((self.vbatadc_lo_vcpuadc_hi_tmp >> 8) & 4095)) * 1400) // 1000
            return getattr(self, '_m_vbat1_dec', None)

        @property
        def spa_dec(self):
            if hasattr(self, '_m_spa_dec'):
                return self._m_spa_dec

            self._m_spa_dec = (self.spa_tmp << 1)
            return getattr(self, '_m_spa_dec', None)

        @property
        def vbus1_dec(self):
            if hasattr(self, '_m_vbus1_dec'):
                return self._m_vbus1_dec

            self._m_vbus1_dec = ((self.vbusadc_vbatadc_hi_tmp >> 4) * 1400) // 1000
            return getattr(self, '_m_vbus1_dec', None)

        @property
        def pwrdul4_dec(self):
            if hasattr(self, '_m_pwrdul4_dec'):
                return self._m_pwrdul4_dec

            self._m_pwrdul4_dec = (self.powerdul455_tmp * 16)
            return getattr(self, '_m_pwrdul4_dec', None)

        @property
        def vbus1_vbat1_dec(self):
            if hasattr(self, '_m_vbus1_vbat1_dec'):
                return self._m_vbus1_vbat1_dec

            self._m_vbus1_vbat1_dec = (self.vbus1_dec - self.vbat1_dec)
            return getattr(self, '_m_vbus1_vbat1_dec', None)

        @property
        def spb_dec(self):
            if hasattr(self, '_m_spb_dec'):
                return self._m_spb_dec

            self._m_spb_dec = (self.spb_tmp << 1)
            return getattr(self, '_m_spb_dec', None)

        @property
        def vcpu_dec(self):
            if hasattr(self, '_m_vcpu_dec'):
                return self._m_vcpu_dec

            self._m_vcpu_dec = (1210 * 4096) // (((self.vbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | (self.vcpuadc_lo_vbus2_tmp >> 12))
            return getattr(self, '_m_vcpu_dec', None)

        @property
        def ipl_dec(self):
            if hasattr(self, '_m_ipl_dec'):
                return self._m_ipl_dec

            self._m_ipl_dec = (self.icpu_lo_ipl_tmp & 4095)
            return getattr(self, '_m_ipl_dec', None)

        @property
        def vbat2_dec(self):
            if hasattr(self, '_m_vbat2_dec'):
                return self._m_vbat2_dec

            self._m_vbat2_dec = ((((self.vbus3_vbat2_hi_tmp << 8) & 3840) | (self.vbat2_lo_ibat_hi_tmp >> 8)) * 4)
            return getattr(self, '_m_vbat2_dec', None)

        @property
        def spd_dec(self):
            if hasattr(self, '_m_spd_dec'):
                return self._m_spd_dec

            self._m_spd_dec = (self.spd_tmp << 1)
            return getattr(self, '_m_spd_dec', None)

        @property
        def vbus2_dec(self):
            if hasattr(self, '_m_vbus2_dec'):
                return self._m_vbus2_dec

            self._m_vbus2_dec = ((self.vcpuadc_lo_vbus2_tmp & 4095) * 4)
            return getattr(self, '_m_vbus2_dec', None)

        @property
        def vbus3_dec(self):
            if hasattr(self, '_m_vbus3_dec'):
                return self._m_vbus3_dec

            self._m_vbus3_dec = ((self.vbus3_vbat2_hi_tmp >> 4) * 4)
            return getattr(self, '_m_vbus3_dec', None)

        @property
        def icpu_dec(self):
            if hasattr(self, '_m_icpu_dec'):
                return self._m_icpu_dec

            self._m_icpu_dec = (((self.ibat_lo_icpu_hi_tmp << 4) & 4080) | (self.icpu_lo_ipl_tmp >> 12))
            return getattr(self, '_m_icpu_dec', None)


    class StatusFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sclock_dec = self._io.read_u4le()
            self.uptime_dec = self._io.read_u2le()
            self.nrun_dec = self._io.read_u2le()
            self.npayload_dec = self._io.read_u1()
            self.nwire_dec = self._io.read_u1()
            self.nbusdrops_lastreset_tmp = self._io.read_u1()
            self.bate_mote_tmp = self._io.read_u1()
            self.ntasksnotexecuted_dec = self._io.read_u1()
            self.antennadeployed_dec = self._io.read_u1()
            self.nexteepromerrors_dec = self._io.read_u1()
            self.failed_task_id_dec = self._io.read_u1()
            self.orbperiod_dec = self._io.read_u1()
            self.strfwd0_dec = self._io.read_u1()
            self.strfwd1_dec = self._io.read_u2le()
            self.strfwd2_dec = self._io.read_u2le()
            self.strfwd3_dec = self._io.read_u1()

        @property
        def transponder_dec(self):
            if hasattr(self, '_m_transponder_dec'):
                return self._m_transponder_dec

            self._m_transponder_dec = (self.bate_mote_tmp & 15)
            return getattr(self, '_m_transponder_dec', None)

        @property
        def nbusdrops_dec(self):
            if hasattr(self, '_m_nbusdrops_dec'):
                return self._m_nbusdrops_dec

            self._m_nbusdrops_dec = (self.nbusdrops_lastreset_tmp >> 4)
            return getattr(self, '_m_nbusdrops_dec', None)

        @property
        def battery_dec(self):
            if hasattr(self, '_m_battery_dec'):
                return self._m_battery_dec

            self._m_battery_dec = (self.bate_mote_tmp >> 4)
            return getattr(self, '_m_battery_dec', None)

        @property
        def last_reset_cause_dec(self):
            if hasattr(self, '_m_last_reset_cause_dec'):
                return self._m_last_reset_cause_dec

            self._m_last_reset_cause_dec = (self.nbusdrops_lastreset_tmp & 15)
            return getattr(self, '_m_last_reset_cause_dec', None)

        @property
        def ntrans_dec(self):
            if hasattr(self, '_m_ntrans_dec'):
                return self._m_ntrans_dec

            self._m_ntrans_dec = (0 if self.nwire_dec <= 25 else (self.nwire_dec - 26))
            return getattr(self, '_m_ntrans_dec', None)


    class FirstHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.type = self._io.read_bits_int_be(4)
            if not self.type >= 1:
                raise kaitaistruct.ValidationLessThanError(1, self.type, self._io, u"/types/first_header/seq/0")
            if not self.type <= 5:
                raise kaitaistruct.ValidationGreaterThanError(5, self.type, self._io, u"/types/first_header/seq/0")
            self.address = self._io.read_bits_int_be(4)
            if not self.address == 8:
                raise kaitaistruct.ValidationNotEqualError(8, self.address, self._io, u"/types/first_header/seq/1")


    class TempstatsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mintpa_tmp = self._io.read_u1()
            self.mintpb_tmp = self._io.read_u1()
            self.mintpc_tmp = self._io.read_u1()
            self.mintpd_tmp = self._io.read_u1()
            self.mintpe_tmp = self._io.read_u1()
            self.minteps_tmp = self._io.read_u1()
            self.minttx_tmp = self._io.read_u1()
            self.minttx2_tmp = self._io.read_u1()
            self.mintrx_tmp = self._io.read_u1()
            self.mintcpu_tmp = self._io.read_u1()
            self.maxtpa_tmp = self._io.read_u1()
            self.maxtpb_tmp = self._io.read_u1()
            self.maxtpc_tmp = self._io.read_u1()
            self.maxtpd_tmp = self._io.read_u1()
            self.maxtpe_tmp = self._io.read_u1()
            self.maxteps_tmp = self._io.read_u1()
            self.maxttx_tmp = self._io.read_u1()
            self.maxttx2_tmp = self._io.read_u1()
            self.maxtrx_tmp = self._io.read_u1()
            self.maxtcpu_tmp = self._io.read_u1()
            self.medtpa_tmp = self._io.read_u1()
            self.medtpb_tmp = self._io.read_u1()
            self.medtpc_tmp = self._io.read_u1()
            self.medtpd_tmp = self._io.read_u1()
            self.medtpe_tmp = self._io.read_u1()
            self.medteps_tmp = self._io.read_u1()
            self.medttx_tmp = self._io.read_u1()
            self.medttx2_tmp = self._io.read_u1()
            self.medtrx_tmp = self._io.read_u1()
            self.medtcpu_tmp = self._io.read_u1()

        @property
        def mintpc_dec(self):
            if hasattr(self, '_m_mintpc_dec'):
                return self._m_mintpc_dec

            self._m_mintpc_dec = ((self.mintpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpc_dec', None)

        @property
        def medtpa_dec(self):
            if hasattr(self, '_m_medtpa_dec'):
                return self._m_medtpa_dec

            self._m_medtpa_dec = ((self.medtpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtpa_dec', None)

        @property
        def maxtrx_dec(self):
            if hasattr(self, '_m_maxtrx_dec'):
                return self._m_maxtrx_dec

            self._m_maxtrx_dec = ((self.maxtrx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtrx_dec', None)

        @property
        def medtrx_dec(self):
            if hasattr(self, '_m_medtrx_dec'):
                return self._m_medtrx_dec

            self._m_medtrx_dec = ((self.medtrx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtrx_dec', None)

        @property
        def maxtpb_dec(self):
            if hasattr(self, '_m_maxtpb_dec'):
                return self._m_maxtpb_dec

            self._m_maxtpb_dec = ((self.maxtpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpb_dec', None)

        @property
        def medtcpu_dec(self):
            if hasattr(self, '_m_medtcpu_dec'):
                return self._m_medtcpu_dec

            self._m_medtcpu_dec = ((self.medtcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtcpu_dec', None)

        @property
        def mintcpu_dec(self):
            if hasattr(self, '_m_mintcpu_dec'):
                return self._m_mintcpu_dec

            self._m_mintcpu_dec = ((self.mintcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintcpu_dec', None)

        @property
        def maxttx2_dec(self):
            if hasattr(self, '_m_maxttx2_dec'):
                return self._m_maxttx2_dec

            self._m_maxttx2_dec = ((self.maxttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxttx2_dec', None)

        @property
        def minteps_dec(self):
            if hasattr(self, '_m_minteps_dec'):
                return self._m_minteps_dec

            self._m_minteps_dec = ((self.minteps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minteps_dec', None)

        @property
        def maxtpd_dec(self):
            if hasattr(self, '_m_maxtpd_dec'):
                return self._m_maxtpd_dec

            self._m_maxtpd_dec = ((self.maxtpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpd_dec', None)

        @property
        def minttx_dec(self):
            if hasattr(self, '_m_minttx_dec'):
                return self._m_minttx_dec

            self._m_minttx_dec = ((self.minttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minttx_dec', None)

        @property
        def medttx2_dec(self):
            if hasattr(self, '_m_medttx2_dec'):
                return self._m_medttx2_dec

            self._m_medttx2_dec = ((self.medttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medttx2_dec', None)

        @property
        def maxtcpu_dec(self):
            if hasattr(self, '_m_maxtcpu_dec'):
                return self._m_maxtcpu_dec

            self._m_maxtcpu_dec = ((self.maxtcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtcpu_dec', None)

        @property
        def minttx2_dec(self):
            if hasattr(self, '_m_minttx2_dec'):
                return self._m_minttx2_dec

            self._m_minttx2_dec = ((self.minttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_minttx2_dec', None)

        @property
        def medttx_dec(self):
            if hasattr(self, '_m_medttx_dec'):
                return self._m_medttx_dec

            self._m_medttx_dec = ((self.medttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medttx_dec', None)

        @property
        def maxtpa_dec(self):
            if hasattr(self, '_m_maxtpa_dec'):
                return self._m_maxtpa_dec

            self._m_maxtpa_dec = ((self.maxtpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpa_dec', None)

        @property
        def medtpb_dec(self):
            if hasattr(self, '_m_medtpb_dec'):
                return self._m_medtpb_dec

            self._m_medtpb_dec = ((self.medtpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtpb_dec', None)

        @property
        def mintrx_dec(self):
            if hasattr(self, '_m_mintrx_dec'):
                return self._m_mintrx_dec

            self._m_mintrx_dec = ((self.mintrx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintrx_dec', None)

        @property
        def mintpd_dec(self):
            if hasattr(self, '_m_mintpd_dec'):
                return self._m_mintpd_dec

            self._m_mintpd_dec = ((self.mintpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpd_dec', None)

        @property
        def medteps_dec(self):
            if hasattr(self, '_m_medteps_dec'):
                return self._m_medteps_dec

            self._m_medteps_dec = ((self.medteps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medteps_dec', None)

        @property
        def mintpa_dec(self):
            if hasattr(self, '_m_mintpa_dec'):
                return self._m_mintpa_dec

            self._m_mintpa_dec = ((self.mintpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpa_dec', None)

        @property
        def maxtpc_dec(self):
            if hasattr(self, '_m_maxtpc_dec'):
                return self._m_maxtpc_dec

            self._m_maxtpc_dec = ((self.maxtpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxtpc_dec', None)

        @property
        def maxteps_dec(self):
            if hasattr(self, '_m_maxteps_dec'):
                return self._m_maxteps_dec

            self._m_maxteps_dec = ((self.maxteps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxteps_dec', None)

        @property
        def mintpb_dec(self):
            if hasattr(self, '_m_mintpb_dec'):
                return self._m_mintpb_dec

            self._m_mintpb_dec = ((self.mintpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_mintpb_dec', None)

        @property
        def maxttx_dec(self):
            if hasattr(self, '_m_maxttx_dec'):
                return self._m_maxttx_dec

            self._m_maxttx_dec = ((self.maxttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_maxttx_dec', None)

        @property
        def medtpd_dec(self):
            if hasattr(self, '_m_medtpd_dec'):
                return self._m_medtpd_dec

            self._m_medtpd_dec = ((self.medtpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtpd_dec', None)

        @property
        def medtpc_dec(self):
            if hasattr(self, '_m_medtpc_dec'):
                return self._m_medtpc_dec

            self._m_medtpc_dec = ((self.medtpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_medtpc_dec', None)


    class PowerstatsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.minvbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.minvbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.minvcpuadc_lo_free_tmp = self._io.read_u1()
            self.minvbus2_tmp = self._io.read_u1()
            self.minvbus3_tmp = self._io.read_u1()
            self.minvbat2_tmp = self._io.read_u1()
            self.minibat_tmp = self._io.read_u1()
            self.minicpu_dec = self._io.read_u1()
            self.minipl_dec = self._io.read_u1()
            self.minpowerdul_dec = self._io.read_u1()
            self.minpowerdul455_dec = self._io.read_u1()
            self.minvdac_dec = self._io.read_u1()
            self.maxvbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.maxvbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.maxvcpuadc_lo_free_tmp = self._io.read_u1()
            self.maxvbus2_tmp = self._io.read_u1()
            self.maxvbus3_tmp = self._io.read_u1()
            self.maxvbat2_tmp = self._io.read_u1()
            self.maxibat_dec = self._io.read_u1()
            self.maxicpu_dec = self._io.read_u1()
            self.maxipl_tmp = self._io.read_u1()
            self.maxpowerdul_dec = self._io.read_u1()
            self.maxpowerdul455_dec = self._io.read_u1()
            self.maxvdac_dec = self._io.read_u1()
            self.medvbusadc_vbatadc_hi_tmp = self._io.read_u2le()
            self.medvbatadc_lo_vcpuadc_hi_tmp = self._io.read_u2le()
            self.medvcpuadc_lo_free_tmp = self._io.read_u1()
            self.ibat_rx_charging_dec = self._io.read_u1()
            self.ibat_rx_discharging_dec = self._io.read_u1()
            self.ibat_tx_low_power_charging_dec = self._io.read_u1()
            self.ibat_tx_low_power_discharging_dec = self._io.read_u1()
            self.ibat_tx_high_power_charging_dec = self._io.read_u1()
            self.ibat_tx_high_power_discharging_dec = self._io.read_u1()
            self.medpowerdul_dec = self._io.read_u1()
            self.medpowerdul455_dec = self._io.read_u1()
            self.medvdac_dec = self._io.read_u1()

        @property
        def maxvbat2_dec(self):
            if hasattr(self, '_m_maxvbat2_dec'):
                return self._m_maxvbat2_dec

            self._m_maxvbat2_dec = ((self.maxvbat2_tmp * 16) * 4)
            return getattr(self, '_m_maxvbat2_dec', None)

        @property
        def maxvcpu_dec(self):
            if hasattr(self, '_m_maxvcpu_dec'):
                return self._m_maxvcpu_dec

            self._m_maxvcpu_dec = ((1210 * 4096) // (((self.maxvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | 1) if self.maxvcpuadc_lo_free_tmp > 4 else (1210 * 4096) // ((self.maxvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080))
            return getattr(self, '_m_maxvcpu_dec', None)

        @property
        def maxvbus3_dec(self):
            if hasattr(self, '_m_maxvbus3_dec'):
                return self._m_maxvbus3_dec

            self._m_maxvbus3_dec = ((self.maxvbus2_tmp * 16) * 4)
            return getattr(self, '_m_maxvbus3_dec', None)

        @property
        def maxipl_dec(self):
            if hasattr(self, '_m_maxipl_dec'):
                return self._m_maxipl_dec

            self._m_maxipl_dec = (self.maxipl_tmp << 2)
            return getattr(self, '_m_maxipl_dec', None)

        @property
        def minvbus1_dec(self):
            if hasattr(self, '_m_minvbus1_dec'):
                return self._m_minvbus1_dec

            self._m_minvbus1_dec = (1400 * (self.minvbusadc_vbatadc_hi_tmp >> 4)) // 1000
            return getattr(self, '_m_minvbus1_dec', None)

        @property
        def maxvbus1_dec(self):
            if hasattr(self, '_m_maxvbus1_dec'):
                return self._m_maxvbus1_dec

            self._m_maxvbus1_dec = (1400 * (self.maxvbusadc_vbatadc_hi_tmp >> 4)) // 1000
            return getattr(self, '_m_maxvbus1_dec', None)

        @property
        def minibat_dec(self):
            if hasattr(self, '_m_minibat_dec'):
                return self._m_minibat_dec

            self._m_minibat_dec = (self.minibat_tmp * -1)
            return getattr(self, '_m_minibat_dec', None)

        @property
        def minvbat2_dec(self):
            if hasattr(self, '_m_minvbat2_dec'):
                return self._m_minvbat2_dec

            self._m_minvbat2_dec = ((self.minvbat2_tmp * 16) * 4)
            return getattr(self, '_m_minvbat2_dec', None)

        @property
        def minvbus2_dec(self):
            if hasattr(self, '_m_minvbus2_dec'):
                return self._m_minvbus2_dec

            self._m_minvbus2_dec = ((self.minvbus2_tmp * 16) * 4)
            return getattr(self, '_m_minvbus2_dec', None)

        @property
        def minvbat1_dec(self):
            if hasattr(self, '_m_minvbat1_dec'):
                return self._m_minvbat1_dec

            self._m_minvbat1_dec = (1400 * (((self.minvbusadc_vbatadc_hi_tmp << 8) & 3840) | ((self.minvbatadc_lo_vcpuadc_hi_tmp >> 8) & 255))) // 1000
            return getattr(self, '_m_minvbat1_dec', None)

        @property
        def maxvbat1_dec(self):
            if hasattr(self, '_m_maxvbat1_dec'):
                return self._m_maxvbat1_dec

            self._m_maxvbat1_dec = (1400 * (((self.maxvbusadc_vbatadc_hi_tmp << 8) & 3840) | ((self.maxvbatadc_lo_vcpuadc_hi_tmp >> 8) & 255))) // 1000
            return getattr(self, '_m_maxvbat1_dec', None)

        @property
        def minvbus3_dec(self):
            if hasattr(self, '_m_minvbus3_dec'):
                return self._m_minvbus3_dec

            self._m_minvbus3_dec = ((self.minvbus2_tmp * 16) * 4)
            return getattr(self, '_m_minvbus3_dec', None)

        @property
        def minvcpu_dec(self):
            if hasattr(self, '_m_minvcpu_dec'):
                return self._m_minvcpu_dec

            self._m_minvcpu_dec = ((1210 * 4096) // (((self.minvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080) | 1) if self.minvcpuadc_lo_free_tmp > 4 else (1210 * 4096) // ((self.minvbatadc_lo_vcpuadc_hi_tmp << 4) & 4080))
            return getattr(self, '_m_minvcpu_dec', None)

        @property
        def maxvbus2_dec(self):
            if hasattr(self, '_m_maxvbus2_dec'):
                return self._m_maxvbus2_dec

            self._m_maxvbus2_dec = ((self.maxvbus2_tmp * 16) * 4)
            return getattr(self, '_m_maxvbus2_dec', None)


    class TempFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.tpa_tmp = self._io.read_u1()
            self.tpb_tmp = self._io.read_u1()
            self.tpc_tmp = self._io.read_u1()
            self.tpd_tmp = self._io.read_u1()
            self.tpe_tmp = self._io.read_u1()
            self.teps_tmp = self._io.read_u1()
            self.ttx_tmp = self._io.read_u1()
            self.ttx2_tmp = self._io.read_u1()
            self.trx_tmp = self._io.read_u1()
            self.tcpu_tmp = self._io.read_u1()

        @property
        def ttx_dec(self):
            if hasattr(self, '_m_ttx_dec'):
                return self._m_ttx_dec

            self._m_ttx_dec = ((self.ttx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_ttx_dec', None)

        @property
        def ttx2_dec(self):
            if hasattr(self, '_m_ttx2_dec'):
                return self._m_ttx2_dec

            self._m_ttx2_dec = ((self.ttx2_tmp / 2.0) - 40.0)
            return getattr(self, '_m_ttx2_dec', None)

        @property
        def teps_dec(self):
            if hasattr(self, '_m_teps_dec'):
                return self._m_teps_dec

            self._m_teps_dec = ((self.teps_tmp / 2.0) - 40.0)
            return getattr(self, '_m_teps_dec', None)

        @property
        def tpc_dec(self):
            if hasattr(self, '_m_tpc_dec'):
                return self._m_tpc_dec

            self._m_tpc_dec = ((self.tpc_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpc_dec', None)

        @property
        def tpd_dec(self):
            if hasattr(self, '_m_tpd_dec'):
                return self._m_tpd_dec

            self._m_tpd_dec = ((self.tpd_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpd_dec', None)

        @property
        def tpb_dec(self):
            if hasattr(self, '_m_tpb_dec'):
                return self._m_tpb_dec

            self._m_tpb_dec = ((self.tpb_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpb_dec', None)

        @property
        def tcpu_dec(self):
            if hasattr(self, '_m_tcpu_dec'):
                return self._m_tcpu_dec

            self._m_tcpu_dec = ((self.tcpu_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tcpu_dec', None)

        @property
        def trx_dec(self):
            if hasattr(self, '_m_trx_dec'):
                return self._m_trx_dec

            self._m_trx_dec = ((self.trx_tmp / 2.0) - 40.0)
            return getattr(self, '_m_trx_dec', None)

        @property
        def tpa_dec(self):
            if hasattr(self, '_m_tpa_dec'):
                return self._m_tpa_dec

            self._m_tpa_dec = ((self.tpa_tmp / 2.0) - 40.0)
            return getattr(self, '_m_tpa_dec', None)


    class CubesatFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.first_header = Hadesd.FirstHeader(self._io, self, self._root)
            _on = self.first_header.type
            if _on == 4:
                self.metadata = Hadesd.PowerstatsFrame(self._io, self, self._root)
            elif _on == 1:
                self.metadata = Hadesd.PowerFrame(self._io, self, self._root)
            elif _on == 3:
                self.metadata = Hadesd.StatusFrame(self._io, self, self._root)
            elif _on == 5:
                self.metadata = Hadesd.TempstatsFrame(self._io, self, self._root)
            elif _on == 2:
                self.metadata = Hadesd.TempFrame(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Neudose(KaitaiStruct):
    """:field dest_callsign: ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_header.ctl
    :field pid: ax25_header.pid
    
    :field csp_hdr_crc: csp_header.crc
    :field csp_hdr_rdp: csp_header.rdp
    :field csp_hdr_xtea: csp_header.xtea
    :field csp_hdr_hmac: csp_header.hmac
    :field csp_hdr_src_port: csp_header.source_port
    :field csp_hdr_dst_port: csp_header.destination_port
    :field csp_hdr_destination: csp_header.destination
    :field csp_hdr_source: csp_header.source
    :field csp_hdr_priority: csp_header.priority
    
    :field last_rx_timestamp_raw: beacon.last_rx_timestamp_raw
    :field packet_version: beacon.packet_version
    :field unix_timestamp: beacon.unix_timestamp
    
    :field cdh_on: beacon.sat_status.cdh_on
    :field eps_on: beacon.sat_status.eps_on
    :field comms_on: beacon.sat_status.comms_on
    :field antenna_on: beacon.sat_status.antenna_on
    :field payload_on: beacon.sat_status.payload_on
    :field mech: beacon.sat_status.mech
    :field thermal: beacon.sat_status.thermal
    :field antenna_deployed: beacon.sat_status.antenna_deployed
    
    :field last_gs_conn_timestamp: beacon.last_gs_conn_timestamp
    
    :field eps_bat_state: beacon.eps_status.bat_state
    :field eps_bat_htr_state: beacon.eps_status.bat_htr_state
    :field eps_bat_htr_mode: beacon.eps_status.bat_htr_mode
    :field eps_last_reset_rsn: beacon.eps_status.last_reset_rsn
    :field eps_gs_wtdg_rst_mark: beacon.eps_status.gs_wtdg_rst_mark
    
    :field eps_uptime: beacon.eps_uptime
    :field eps_vbat: beacon.eps_vbat
    :field eps_bat_chrge_curr: beacon.eps_bat_chrge_curr
    :field eps_bat_dischrge_curr: beacon.eps_bat_dischrge_curr
    
    :field eps_mppt_conv1_temp: beacon.eps_temp.mppt_conv1
    :field eps_mppt_conv2_temp: beacon.eps_temp.mppt_conv2
    :field eps_mppt_conv3_temp: beacon.eps_temp.mppt_conv3
    :field eps_out_conv_3v3_temp: beacon.eps_temp.out_conv_3v3
    :field eps_out_conv_5v0_temp: beacon.eps_temp.out_conv_5v0
    :field eps_battery_pack_temp: beacon.eps_temp.battery_pack
    
    :field eps_solar_panel_y_n_curr: beacon.eps_solar_panel_curr.y_n
    :field eps_solar_panel_y_p_curr: beacon.eps_solar_panel_curr.y_p
    :field eps_solar_panel_x_n_curr: beacon.eps_solar_panel_curr.x_n
    :field eps_solar_panel_x_p_curr: beacon.eps_solar_panel_curr.x_p
    :field eps_solar_panel_z_n_curr: beacon.eps_solar_panel_curr.z_n
    :field eps_solar_panel_z_p_curr: beacon.eps_solar_panel_curr.z_p
    
    :field eps_cdh_channel_curr_out: beacon.eps_channel_curr_out.cdh
    :field eps_comm_3v3_channel_curr_out: beacon.eps_channel_curr_out.comm_3v3
    :field eps_comm_5v0_channel_curr_out: beacon.eps_channel_curr_out.comm_5v0
    :field eps_ant_channel_curr_out: beacon.eps_channel_curr_out.ant
    :field eps_pld_channel_curr_out: beacon.eps_channel_curr_out.pld
    
    :field cdh_curr_state: beacon.cdh_curr_state
    :field cdh_prev_state: beacon.cdh_prev_state
    
    :field cdh_reset_cause: beacon.cdh_boot_reset_cause.reset_cause
    :field cdh_boot_cause: beacon.cdh_boot_reset_cause.boot_cause
    
    :field cdh_uptime: beacon.cdh_uptime
    :field cdh_temp_mcu_raw: beacon.cdh_temp_mcu_raw
    :field cdh_temp_ram_raw: beacon.cdh_temp_ram_raw
    
    :field comms_rtsm_state: beacon.comms_status.rtsm_state
    :field comms_rst_reason: beacon.comms_status.rst_reason
    :field comms_boot_img_bank: beacon.comms_status.boot_img_bank
    
    :field comms_uptime_raw: beacon.comms_uptime_raw
    :field comms_ina233_pa_curr_raw: beacon.comms_ina233_pa_curr_raw
    :field comms_ad7294_pa_curr_raw: beacon.comms_ad7294_pa_curr_raw
    :field comms_ad7294_gate_volt_raw: beacon.comms_ad7294_gate_volt_raw
    :field comms_cc1125_rssi_raw: beacon.comms_cc1125_rssi_raw
    
    :field comms_lna_therm_temp: beacon.comms_temp.lna_therm
    :field comms_lna_diode_temp: beacon.comms_temp.lna_diode
    :field comms_stm32_internal_temp: beacon.comms_temp.stm32_internal
    :field comms_cc1125_uhf_temp: beacon.comms_temp.cc1125_uhf
    :field comms_cc1125_vhf_temp: beacon.comms_temp.cc1125_vhf
    :field comms_pa_therm_temp: beacon.comms_temp.pa_therm
    :field comms_pa_diode_temp: beacon.comms_temp.pa_diode
    :field comms_pa_therm_strap_temp: beacon.comms_temp.pa_therm_strap
    :field comms_ad7294_internal_temp: beacon.comms_temp.ad7294_internal
    
    :field ant_deployment_status: beacon.ant_deployment_status
    :field ant_prev_isis_status: beacon.ant_prev_isis_status
    :field pld_status: beacon.pld_status
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_header = Neudose.Ax25HeaderT(self._io, self, self._root)
        self.csp_header = Neudose.CspHeaderT(self._io, self, self._root)
        self.beacon = Neudose.BeaconT(self._io, self, self._root)

    class EpsSolarPanelCurrT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_solar_panel_curr = self._io.read_bytes(12)

        @property
        def z_p(self):
            if hasattr(self, '_m_z_p'):
                return self._m_z_p

            self._m_z_p = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 10) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 11))
            return getattr(self, '_m_z_p', None)

        @property
        def x_p(self):
            if hasattr(self, '_m_x_p'):
                return self._m_x_p

            self._m_x_p = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 6) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 7))
            return getattr(self, '_m_x_p', None)

        @property
        def y_p(self):
            if hasattr(self, '_m_y_p'):
                return self._m_y_p

            self._m_y_p = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 2) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 3))
            return getattr(self, '_m_y_p', None)

        @property
        def x_n(self):
            if hasattr(self, '_m_x_n'):
                return self._m_x_n

            self._m_x_n = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 4) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 5))
            return getattr(self, '_m_x_n', None)

        @property
        def y_n(self):
            if hasattr(self, '_m_y_n'):
                return self._m_y_n

            self._m_y_n = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 0) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 1))
            return getattr(self, '_m_y_n', None)

        @property
        def z_n(self):
            if hasattr(self, '_m_z_n'):
                return self._m_z_n

            self._m_z_n = ((KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 8) << 8) + KaitaiStream.byte_array_index(self.eps_solar_panel_curr, 9))
            return getattr(self, '_m_z_n', None)


    class SatStatusT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sat_status = self._io.read_u1()

        @property
        def comms_on(self):
            if hasattr(self, '_m_comms_on'):
                return self._m_comms_on

            self._m_comms_on = ((self.sat_status >> 5) & 1)
            return getattr(self, '_m_comms_on', None)

        @property
        def antenna_on(self):
            if hasattr(self, '_m_antenna_on'):
                return self._m_antenna_on

            self._m_antenna_on = ((self.sat_status >> 4) & 1)
            return getattr(self, '_m_antenna_on', None)

        @property
        def cdh_on(self):
            if hasattr(self, '_m_cdh_on'):
                return self._m_cdh_on

            self._m_cdh_on = ((self.sat_status >> 7) & 1)
            return getattr(self, '_m_cdh_on', None)

        @property
        def mech(self):
            if hasattr(self, '_m_mech'):
                return self._m_mech

            self._m_mech = ((self.sat_status >> 2) & 1)
            return getattr(self, '_m_mech', None)

        @property
        def antenna_deployed(self):
            if hasattr(self, '_m_antenna_deployed'):
                return self._m_antenna_deployed

            self._m_antenna_deployed = ((self.sat_status >> 0) & 1)
            return getattr(self, '_m_antenna_deployed', None)

        @property
        def thermal(self):
            if hasattr(self, '_m_thermal'):
                return self._m_thermal

            self._m_thermal = ((self.sat_status >> 1) & 1)
            return getattr(self, '_m_thermal', None)

        @property
        def eps_on(self):
            if hasattr(self, '_m_eps_on'):
                return self._m_eps_on

            self._m_eps_on = ((self.sat_status >> 6) & 1)
            return getattr(self, '_m_eps_on', None)

        @property
        def payload_on(self):
            if hasattr(self, '_m_payload_on'):
                return self._m_payload_on

            self._m_payload_on = ((self.sat_status >> 3) & 1)
            return getattr(self, '_m_payload_on', None)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Ax25HeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Neudose.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Neudose.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Neudose.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Neudose.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class CommsStatusT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.comms_status = self._io.read_u1()

        @property
        def rtsm_state(self):
            if hasattr(self, '_m_rtsm_state'):
                return self._m_rtsm_state

            self._m_rtsm_state = ((self.comms_status >> 5) & 7)
            return getattr(self, '_m_rtsm_state', None)

        @property
        def rst_reason(self):
            if hasattr(self, '_m_rst_reason'):
                return self._m_rst_reason

            self._m_rst_reason = ((self.comms_status >> 2) & 7)
            return getattr(self, '_m_rst_reason', None)

        @property
        def boot_img_bank(self):
            if hasattr(self, '_m_boot_img_bank'):
                return self._m_boot_img_bank

            self._m_boot_img_bank = ((self.comms_status >> 0) & 3)
            return getattr(self, '_m_boot_img_bank', None)


    class EpsChannelCurrOutT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_channel_curr_out = self._io.read_bytes(10)

        @property
        def cdh(self):
            if hasattr(self, '_m_cdh'):
                return self._m_cdh

            self._m_cdh = ((KaitaiStream.byte_array_index(self.eps_channel_curr_out, 0) << 8) + KaitaiStream.byte_array_index(self.eps_channel_curr_out, 1))
            return getattr(self, '_m_cdh', None)

        @property
        def ant(self):
            if hasattr(self, '_m_ant'):
                return self._m_ant

            self._m_ant = ((KaitaiStream.byte_array_index(self.eps_channel_curr_out, 6) << 8) + KaitaiStream.byte_array_index(self.eps_channel_curr_out, 7))
            return getattr(self, '_m_ant', None)

        @property
        def comm_3v3(self):
            if hasattr(self, '_m_comm_3v3'):
                return self._m_comm_3v3

            self._m_comm_3v3 = ((KaitaiStream.byte_array_index(self.eps_channel_curr_out, 2) << 8) + KaitaiStream.byte_array_index(self.eps_channel_curr_out, 3))
            return getattr(self, '_m_comm_3v3', None)

        @property
        def comm_5v0(self):
            if hasattr(self, '_m_comm_5v0'):
                return self._m_comm_5v0

            self._m_comm_5v0 = ((KaitaiStream.byte_array_index(self.eps_channel_curr_out, 4) << 8) + KaitaiStream.byte_array_index(self.eps_channel_curr_out, 5))
            return getattr(self, '_m_comm_5v0', None)

        @property
        def pld(self):
            if hasattr(self, '_m_pld'):
                return self._m_pld

            self._m_pld = ((KaitaiStream.byte_array_index(self.eps_channel_curr_out, 8) << 8) + KaitaiStream.byte_array_index(self.eps_channel_curr_out, 9))
            return getattr(self, '_m_pld', None)


    class CommsTempT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.lna_therm = self._io.read_s1()
            self.lna_diode = self._io.read_s1()
            self.stm32_internal = self._io.read_s1()
            self.cc1125_uhf = self._io.read_s1()
            self.cc1125_vhf = self._io.read_s1()
            self.pa_therm = self._io.read_s1()
            self.pa_diode = self._io.read_s1()
            self.pa_therm_strap = self._io.read_s1()
            self.ad7294_internal = self._io.read_s1()


    class BeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.last_rx_timestamp_raw = self._io.read_u4be()
            self.packet_version = self._io.read_u1()
            self.unix_timestamp = self._io.read_u4be()
            self.sat_status = Neudose.SatStatusT(self._io, self, self._root)
            self.last_gs_conn_timestamp = self._io.read_u4be()
            self.eps_status = Neudose.EpsStatusT(self._io, self, self._root)
            self.eps_uptime = self._io.read_u4be()
            self.eps_vbat = self._io.read_u2be()
            self.eps_bat_chrge_curr = self._io.read_u2be()
            self.eps_bat_dischrge_curr = self._io.read_u2be()
            self.eps_temp = Neudose.EpsTempT(self._io, self, self._root)
            self.eps_solar_panel_curr = Neudose.EpsSolarPanelCurrT(self._io, self, self._root)
            self.eps_channel_curr_out = Neudose.EpsChannelCurrOutT(self._io, self, self._root)
            self.cdh_curr_state = self._io.read_u1()
            self.cdh_prev_state = self._io.read_u1()
            self.cdh_boot_reset_cause = Neudose.CdhBootResetCauseT(self._io, self, self._root)
            self.cdh_uptime = self._io.read_u4be()
            self.cdh_temp_mcu_raw = self._io.read_s2be()
            self.cdh_temp_ram_raw = self._io.read_s2be()
            self.comms_status = Neudose.CommsStatusT(self._io, self, self._root)
            self.comms_uptime_raw = self._io.read_u4be()
            self.comms_ina233_pa_curr_raw = self._io.read_u2be()
            self.comms_ad7294_pa_curr_raw = self._io.read_u2be()
            self.comms_ad7294_gate_volt_raw = self._io.read_u2be()
            self.comms_cc1125_rssi_raw = self._io.read_u2be()
            self.comms_temp = Neudose.CommsTempT(self._io, self, self._root)
            self.ant_deployment_status = self._io.read_u1()
            self.ant_prev_isis_status = self._io.read_u2be()
            self.pld_status = self._io.read_u1()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class EpsTempT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mppt_conv1 = self._io.read_s1()
            self.mppt_conv2 = self._io.read_s1()
            self.mppt_conv3 = self._io.read_s1()
            self.out_conv_3v3 = self._io.read_s1()
            self.out_conv_5v0 = self._io.read_s1()
            self.battery_pack = self._io.read_s1()


    class CspHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_length = self._io.read_u2be()
            self.csp_header_raw = self._io.read_u4be()

        @property
        def source(self):
            if hasattr(self, '_m_source'):
                return self._m_source

            self._m_source = ((self.csp_header_raw >> 25) & 31)
            return getattr(self, '_m_source', None)

        @property
        def source_port(self):
            if hasattr(self, '_m_source_port'):
                return self._m_source_port

            self._m_source_port = ((self.csp_header_raw >> 8) & 63)
            return getattr(self, '_m_source_port', None)

        @property
        def destination_port(self):
            if hasattr(self, '_m_destination_port'):
                return self._m_destination_port

            self._m_destination_port = ((self.csp_header_raw >> 14) & 63)
            return getattr(self, '_m_destination_port', None)

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp

            self._m_rdp = ((self.csp_header_raw & 2) >> 1)
            return getattr(self, '_m_rdp', None)

        @property
        def destination(self):
            if hasattr(self, '_m_destination'):
                return self._m_destination

            self._m_destination = ((self.csp_header_raw >> 20) & 31)
            return getattr(self, '_m_destination', None)

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority

            self._m_priority = (self.csp_header_raw >> 30)
            return getattr(self, '_m_priority', None)

        @property
        def reserved(self):
            if hasattr(self, '_m_reserved'):
                return self._m_reserved

            self._m_reserved = ((self.csp_header_raw >> 4) & 15)
            return getattr(self, '_m_reserved', None)

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea

            self._m_xtea = ((self.csp_header_raw & 4) >> 2)
            return getattr(self, '_m_xtea', None)

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac

            self._m_hmac = ((self.csp_header_raw & 8) >> 3)
            return getattr(self, '_m_hmac', None)

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc

            self._m_crc = (self.csp_header_raw & 1)
            return getattr(self, '_m_crc', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Neudose.Callsign(_io__raw_callsign_ror, self, self._root)


    class CdhBootResetCauseT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cdh_boot_reset_cause = self._io.read_u1()

        @property
        def boot_cause(self):
            if hasattr(self, '_m_boot_cause'):
                return self._m_boot_cause

            self._m_boot_cause = ((self.cdh_boot_reset_cause >> 4) & 15)
            return getattr(self, '_m_boot_cause', None)

        @property
        def reset_cause(self):
            if hasattr(self, '_m_reset_cause'):
                return self._m_reset_cause

            self._m_reset_cause = ((self.cdh_boot_reset_cause >> 0) & 15)
            return getattr(self, '_m_reset_cause', None)


    class EpsStatusT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.eps_status = self._io.read_u1()

        @property
        def gs_wtdg_rst_mark(self):
            if hasattr(self, '_m_gs_wtdg_rst_mark'):
                return self._m_gs_wtdg_rst_mark

            self._m_gs_wtdg_rst_mark = ((self.eps_status >> 0) & 1)
            return getattr(self, '_m_gs_wtdg_rst_mark', None)

        @property
        def bat_htr_mode(self):
            if hasattr(self, '_m_bat_htr_mode'):
                return self._m_bat_htr_mode

            self._m_bat_htr_mode = ((self.eps_status >> 4) & 1)
            return getattr(self, '_m_bat_htr_mode', None)

        @property
        def bat_htr_state(self):
            if hasattr(self, '_m_bat_htr_state'):
                return self._m_bat_htr_state

            self._m_bat_htr_state = ((self.eps_status >> 5) & 1)
            return getattr(self, '_m_bat_htr_state', None)

        @property
        def bat_state(self):
            if hasattr(self, '_m_bat_state'):
                return self._m_bat_state

            self._m_bat_state = ((self.eps_status >> 6) & 3)
            return getattr(self, '_m_bat_state', None)

        @property
        def last_reset_rsn(self):
            if hasattr(self, '_m_last_reset_rsn'):
                return self._m_last_reset_rsn

            self._m_last_reset_rsn = ((self.eps_status >> 1) & 7)
            return getattr(self, '_m_last_reset_rsn', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Ax25frames(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Ax25frames.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Ax25frames.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Ax25frames.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Ax25frames.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Ax25frames.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Ax25frames.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Ax25frames.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Ax25frames.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Ax25frames.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Ax25frames.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Ax25frames.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Ax25frames.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Ax25frames.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Csim(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field l0_status: ax25_frame.payload.ax25_info.l0_status
    :field l0_acpt_cnt: ax25_frame.payload.ax25_info.l0_acpt_cnt
    :field l0_rjct_cnt: ax25_frame.payload.ax25_info.l0_rjct_cnt
    :field hw_sec_cnt: ax25_frame.payload.ax25_info.hw_sec_cnt
    :field time_tag: ax25_frame.payload.ax25_info.time_tag
    :field pld_tlm_ack_cnt: ax25_frame.payload.ax25_info.pld_tlm_ack_cnt
    :field pld_cmd_cnt: ax25_frame.payload.ax25_info.pld_cmd_cnt
    :field pld_tlm_to_cnt: ax25_frame.payload.ax25_info.pld_tlm_to_cnt
    :field pld_tlm_nak_cnt: ax25_frame.payload.ax25_info.pld_tlm_nak_cnt
    :field cmd_status: ax25_frame.payload.ax25_info.cmd_status
    :field realtime_cmd_accept_count: ax25_frame.payload.ax25_info.realtime_cmd_accept_count
    :field realtime_cmd_reject_count: ax25_frame.payload.ax25_info.realtime_cmd_reject_count
    :field stored_cmd_accept_cnt: ax25_frame.payload.ax25_info.stored_cmd_accept_cnt
    :field stored_cmd_reject_cnt: ax25_frame.payload.ax25_info.stored_cmd_reject_cnt
    :field macros_status_1: ax25_frame.payload.ax25_info.macros_status_1
    :field scrub_status_overall: ax25_frame.payload.ax25_info.scrub_status_overall
    :field scrub_count: ax25_frame.payload.ax25_info.scrub_count
    :field image_booted: ax25_frame.payload.ax25_info.image_booted
    :field image_auto_failover: ax25_frame.payload.ax25_info.image_auto_failover
    :field tai_seconds: ax25_frame.payload.ax25_info.tai_seconds
    :field time_valid: ax25_frame.payload.ax25_info.time_valid
    :field position_wrt_eci1: ax25_frame.payload.ax25_info.position_wrt_eci1
    :field position_wrt_eci2: ax25_frame.payload.ax25_info.position_wrt_eci2
    :field position_wrt_eci3: ax25_frame.payload.ax25_info.position_wrt_eci3
    :field velocity_wrt_eci1: ax25_frame.payload.ax25_info.velocity_wrt_eci1
    :field velocity_wrt_eci2: ax25_frame.payload.ax25_info.velocity_wrt_eci2
    :field velocity_wrt_eci3: ax25_frame.payload.ax25_info.velocity_wrt_eci3
    :field nadir_vector_body1: ax25_frame.payload.ax25_info.nadir_vector_body1
    :field nadir_vector_body2: ax25_frame.payload.ax25_info.nadir_vector_body2
    :field nadir_vector_body3: ax25_frame.payload.ax25_info.nadir_vector_body3
    :field sun_vector_body1: ax25_frame.payload.ax25_info.sun_vector_body1
    :field sun_vector_body2: ax25_frame.payload.ax25_info.sun_vector_body2
    :field sun_vector_body3: ax25_frame.payload.ax25_info.sun_vector_body3
    :field sun_position_wrt_eci1: ax25_frame.payload.ax25_info.sun_position_wrt_eci1
    :field sun_position_wrt_eci2: ax25_frame.payload.ax25_info.sun_position_wrt_eci2
    :field sun_position_wrt_eci3: ax25_frame.payload.ax25_info.sun_position_wrt_eci3
    :field moon_position_wrt_eci1: ax25_frame.payload.ax25_info.moon_position_wrt_eci1
    :field moon_position_wrt_eci2: ax25_frame.payload.ax25_info.moon_position_wrt_eci2
    :field moon_position_wrt_eci3: ax25_frame.payload.ax25_info.moon_position_wrt_eci3
    :field refs_valid: ax25_frame.payload.ax25_info.refs_valid
    :field esm_valid: ax25_frame.payload.ax25_info.esm_valid
    :field run_low_rate_task: ax25_frame.payload.ax25_info.run_low_rate_task
    :field q_body_wrt_eci1: ax25_frame.payload.ax25_info.q_body_wrt_eci1
    :field q_body_wrt_eci2: ax25_frame.payload.ax25_info.q_body_wrt_eci2
    :field q_body_wrt_eci3: ax25_frame.payload.ax25_info.q_body_wrt_eci3
    :field q_body_wrt_eci4: ax25_frame.payload.ax25_info.q_body_wrt_eci4
    :field body_rate1_dps: ax25_frame.payload.ax25_info.body_rate1_dps
    :field body_rate2_dps: ax25_frame.payload.ax25_info.body_rate2_dps
    :field body_rate3_dps: ax25_frame.payload.ax25_info.body_rate3_dps
    :field bad_att_timer: ax25_frame.payload.ax25_info.bad_att_timer
    :field bad_rate_timer: ax25_frame.payload.ax25_info.bad_rate_timer
    :field reinit_count: ax25_frame.payload.ax25_info.reinit_count
    :field gnc_status_1: ax25_frame.payload.ax25_info.gnc_status_1
    :field hr_cycle_safe_mode: ax25_frame.payload.ax25_info.hr_cycle_safe_mode
    :field rotisserie_rate_dps: ax25_frame.payload.ax25_info.rotisserie_rate_dps
    :field adcs_mode: ax25_frame.payload.ax25_info.adcs_mode
    :field gnc_status_2: ax25_frame.payload.ax25_info.gnc_status_2
    :field filtered_speed_rpm1: ax25_frame.payload.ax25_info.filtered_speed_rpm1
    :field filtered_speed_rpm2: ax25_frame.payload.ax25_info.filtered_speed_rpm2
    :field filtered_speed_rpm3: ax25_frame.payload.ax25_info.filtered_speed_rpm3
    :field operating_mode1: ax25_frame.payload.ax25_info.operating_mode1
    :field operating_mode2: ax25_frame.payload.ax25_info.operating_mode2
    :field operating_mode3: ax25_frame.payload.ax25_info.operating_mode3
    :field operating_mode: ax25_frame.payload.ax25_info.operating_mode
    :field star_id_step: ax25_frame.payload.ax25_info.star_id_step
    :field att_status: ax25_frame.payload.ax25_info.att_status
    :field det_timeout_count: ax25_frame.payload.ax25_info.det_timeout_count
    :field num_attitude_stars: ax25_frame.payload.ax25_info.num_attitude_stars
    :field position_error1: ax25_frame.payload.ax25_info.position_error1
    :field position_error2: ax25_frame.payload.ax25_info.position_error2
    :field position_error3: ax25_frame.payload.ax25_info.position_error3
    :field eigen_error: ax25_frame.payload.ax25_info.eigen_error
    :field time_into_search: ax25_frame.payload.ax25_info.time_into_search
    :field wait_timer: ax25_frame.payload.ax25_info.wait_timer
    :field sun_point_angle_error: ax25_frame.payload.ax25_info.sun_point_angle_error
    :field sun_point_state: ax25_frame.payload.ax25_info.sun_point_state
    :field momentum_vector_body1: ax25_frame.payload.ax25_info.momentum_vector_body1
    :field momentum_vector_body2: ax25_frame.payload.ax25_info.momentum_vector_body2
    :field momentum_vector_body3: ax25_frame.payload.ax25_info.momentum_vector_body3
    :field total_momentum_mag: ax25_frame.payload.ax25_info.total_momentum_mag
    :field duty_cycle1: ax25_frame.payload.ax25_info.duty_cycle1
    :field duty_cycle2: ax25_frame.payload.ax25_info.duty_cycle2
    :field duty_cycle3: ax25_frame.payload.ax25_info.duty_cycle3
    :field torque_rod_mode1: ax25_frame.payload.ax25_info.torque_rod_mode1
    :field torque_rod_mode2: ax25_frame.payload.ax25_info.torque_rod_mode2
    :field torque_rod_mode3: ax25_frame.payload.ax25_info.torque_rod_mode3
    :field mag_source_used: ax25_frame.payload.ax25_info.mag_source_used
    :field momentum_vector_valid: ax25_frame.payload.ax25_info.momentum_vector_valid
    :field sun_vector_body1_meas: ax25_frame.payload.ax25_info.sun_vector_body1_meas
    :field sun_vector_body2_meas: ax25_frame.payload.ax25_info.sun_vector_body2_meas
    :field sun_vector_body3_meas: ax25_frame.payload.ax25_info.sun_vector_body3_meas
    :field sun_vector_status: ax25_frame.payload.ax25_info.sun_vector_status
    :field css_invalid_count: ax25_frame.payload.ax25_info.css_invalid_count
    :field sun_sensor_used: ax25_frame.payload.ax25_info.sun_sensor_used
    :field mag_vector_body1: ax25_frame.payload.ax25_info.mag_vector_body1
    :field mag_vector_body2: ax25_frame.payload.ax25_info.mag_vector_body2
    :field mag_vector_body3: ax25_frame.payload.ax25_info.mag_vector_body3
    :field mag_invalid_count: ax25_frame.payload.ax25_info.mag_invalid_count
    :field mag_vector_valid: ax25_frame.payload.ax25_info.mag_vector_valid
    :field mag_sensor_used: ax25_frame.payload.ax25_info.mag_sensor_used
    :field imu_invalid_count: ax25_frame.payload.ax25_info.imu_invalid_count
    :field new_packet_count: ax25_frame.payload.ax25_info.new_packet_count
    :field imu_vector_valid: ax25_frame.payload.ax25_info.imu_vector_valid
    :field hr_run_count: ax25_frame.payload.ax25_info.hr_run_count
    :field hr_exec_time_ms1: ax25_frame.payload.ax25_info.hr_exec_time_ms1
    :field hr_exec_time_ms2: ax25_frame.payload.ax25_info.hr_exec_time_ms2
    :field hr_exec_time_ms3: ax25_frame.payload.ax25_info.hr_exec_time_ms3
    :field hr_exec_time_ms4: ax25_frame.payload.ax25_info.hr_exec_time_ms4
    :field hr_exec_time_ms5: ax25_frame.payload.ax25_info.hr_exec_time_ms5
    :field payload_sec_since_last_tlm: ax25_frame.payload.ax25_info.payload_sec_since_last_tlm
    :field payload_tlm_rx_count: ax25_frame.payload.ax25_info.payload_tlm_rx_count
    :field payload_tlm_ack_count: ax25_frame.payload.ax25_info.payload_tlm_ack_count
    :field payload_tlm_nak_count: ax25_frame.payload.ax25_info.payload_tlm_nak_count
    :field voltage_12p0: ax25_frame.payload.ax25_info.voltage_12p0
    :field voltage_8p0: ax25_frame.payload.ax25_info.voltage_8p0
    :field voltage_5p0: ax25_frame.payload.ax25_info.voltage_5p0
    :field voltage_3p3: ax25_frame.payload.ax25_info.voltage_3p3
    :field det_temp: ax25_frame.payload.ax25_info.det_temp
    :field det2_temp: ax25_frame.payload.ax25_info.det2_temp
    :field box1_temp: ax25_frame.payload.ax25_info.box1_temp
    :field imu_temp: ax25_frame.payload.ax25_info.imu_temp
    :field motor1_temp: ax25_frame.payload.ax25_info.motor1_temp
    :field motor2_temp: ax25_frame.payload.ax25_info.motor2_temp
    :field motor3_temp: ax25_frame.payload.ax25_info.motor3_temp
    :field bus_voltage: ax25_frame.payload.ax25_info.bus_voltage
    :field battery_voltage: ax25_frame.payload.ax25_info.battery_voltage
    :field battery_current: ax25_frame.payload.ax25_info.battery_current
    :field battery1_temp: ax25_frame.payload.ax25_info.battery1_temp
    :field battery2_temp: ax25_frame.payload.ax25_info.battery2_temp
    :field user_analog1: ax25_frame.payload.ax25_info.user_analog1
    :field user_analog2: ax25_frame.payload.ax25_info.user_analog2
    :field operating_mode: ax25_frame.payload.ax25_info.operating_mode
    :field star_id_step: ax25_frame.payload.ax25_info.star_id_step
    :field att_status: ax25_frame.payload.ax25_info.att_status
    :field det_timeout_count: ax25_frame.payload.ax25_info.det_timeout_count
    :field num_attitude_stars: ax25_frame.payload.ax25_info.num_attitude_stars
    :field cycles_since_crc_data: ax25_frame.payload.ax25_info.cycles_since_crc_data
    :field gps_lock_count: ax25_frame.payload.ax25_info.gps_lock_count
    :field gps_valid: ax25_frame.payload.ax25_info.gps_valid
    :field gps_enabled: ax25_frame.payload.ax25_info.gps_enabled
    :field macros_status_2: ax25_frame.payload.ax25_info.macros_status_2
    :field sd_minute_cur: ax25_frame.payload.ax25_info.sd_minute_cur
    :field sd_percent_used_total: ax25_frame.payload.ax25_info.sd_percent_used_total
    :field sd_percent_used_fsw: ax25_frame.payload.ax25_info.sd_percent_used_fsw
    :field sd_percent_used_soh: ax25_frame.payload.ax25_info.sd_percent_used_soh
    :field sd_percent_used_line: ax25_frame.payload.ax25_info.sd_percent_used_line
    :field sd_percent_used_tbl: ax25_frame.payload.ax25_info.sd_percent_used_tbl
    :field sd_percent_used_pay: ax25_frame.payload.ax25_info.sd_percent_used_pay
    :field sdr_tx_frames: ax25_frame.payload.ax25_info.sdr_tx_frames
    :field sdr_rx_frames: ax25_frame.payload.ax25_info.sdr_rx_frames
    :field sdr_tx: ax25_frame.payload.ax25_info.sdr_tx
    :field sdr_tx_power: ax25_frame.payload.ax25_info.sdr_tx_power
    :field sdr_rx_lock: ax25_frame.payload.ax25_info.sdr_rx_lock
    :field sdr_rx_power: ax25_frame.payload.ax25_info.sdr_rx_power
    :field sdr_rx_freq_offset: ax25_frame.payload.ax25_info.sdr_rx_freq_offset
    :field sdr_temp: ax25_frame.payload.ax25_info.sdr_temp
    :field sdr_comm_error: ax25_frame.payload.ax25_info.sdr_comm_error
    :field sq_channel: ax25_frame.payload.ax25_info.sq_channel
    :field sq_trap_count: ax25_frame.payload.ax25_info.sq_trap_count
    :field sq_temp: ax25_frame.payload.ax25_info.sq_temp
    :field aid_status: ax25_frame.payload.ax25_info.aid_status
    :field star_id_status: ax25_frame.payload.ax25_info.star_id_status
    :field power_status: ax25_frame.payload.ax25_info.power_status
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Csim.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Csim.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Csim.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Csim.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Csim.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Csim.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Csim.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Csim.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Csim.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Csim.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Csim.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Csim.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Csim.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self.frame_length
            if _on == 131:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Csim.BeaconShort(_io__raw_ax25_info, self, self._root)
            elif _on == 272:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Csim.BeaconLong(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()

        @property
        def frame_length(self):
            if hasattr(self, '_m_frame_length'):
                return self._m_frame_length

            self._m_frame_length = self._io.size()
            return getattr(self, '_m_frame_length', None)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class BeaconShort(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.offset_0 = []
            for i in range(4):
                self.offset_0.append(self._io.read_u1())

            self.imu_invalid_count = self._io.read_u2be()
            self.new_packet_count = self._io.read_u1()
            self.imu_vector_valid = self._io.read_u1()
            self.hr_run_count = self._io.read_u4be()
            self.hr_exec_time_ms1 = self._io.read_u1()
            self.hr_exec_time_ms2 = self._io.read_u1()
            self.hr_exec_time_ms3 = self._io.read_u1()
            self.hr_exec_time_ms4 = self._io.read_u1()
            self.hr_exec_time_ms5 = self._io.read_u1()
            self.payload_sec_since_last_tlm = self._io.read_u4be()
            self.payload_tlm_rx_count = self._io.read_u2be()
            self.payload_tlm_ack_count = self._io.read_u2be()
            self.payload_tlm_nak_count = self._io.read_u2be()
            self.voltage_12p0 = self._io.read_u1()
            self.voltage_8p0 = self._io.read_u1()
            self.voltage_5p0 = self._io.read_u1()
            self.voltage_3p3 = self._io.read_u1()
            self.det_temp = self._io.read_s1()
            self.det2_temp = self._io.read_s1()
            self.box1_temp = self._io.read_s2be()
            self.imu_temp = self._io.read_s2be()
            self.motor1_temp = self._io.read_s2be()
            self.motor2_temp = self._io.read_s2be()
            self.motor3_temp = self._io.read_s2be()
            self.bus_voltage = self._io.read_s2be()
            self.battery_voltage = self._io.read_u2be()
            self.battery_current = self._io.read_s2be()
            self.battery1_temp = self._io.read_s2be()
            self.battery2_temp = self._io.read_s2be()
            self.user_analog1 = self._io.read_s4be()
            self.user_analog2 = self._io.read_s4be()
            self.operating_mode = self._io.read_u1()
            self.star_id_step = self._io.read_u1()
            self.att_status = self._io.read_u1()
            self.det_timeout_count = self._io.read_u2be()
            self.num_attitude_stars = self._io.read_u1()
            self.cycles_since_crc_data = self._io.read_u4be()
            self.gps_lock_count = self._io.read_u2be()
            self.gps_valid = self._io.read_u1()
            self.gps_enabled = self._io.read_u1()
            self.macros_status_2 = self._io.read_u2be()
            self.sd_minute_cur = self._io.read_u4be()
            self.sd_percent_used_total = self._io.read_u1()
            self.sd_percent_used_fsw = self._io.read_u1()
            self.sd_percent_used_soh = self._io.read_u1()
            self.sd_percent_used_line = self._io.read_u1()
            self.sd_percent_used_tbl = self._io.read_u1()
            self.sd_percent_used_pay = self._io.read_u1()
            self.sdr_tx_frames = self._io.read_u4be()
            self.sdr_rx_frames = self._io.read_u4be()
            self.sdr_tx = self._io.read_u1()
            self.sdr_tx_power = self._io.read_s1()
            self.sdr_rx_lock = self._io.read_u1()
            self.sdr_rx_power = self._io.read_s1()
            self.sdr_rx_freq_offset = self._io.read_s4be()
            self.sdr_temp = self._io.read_s1()
            self.sdr_comm_error = self._io.read_u1()
            self.sq_channel = self._io.read_s1()
            self.sq_trap_count = self._io.read_u1()
            self.sq_temp = self._io.read_u1()
            self.aid_status = self._io.read_u2be()
            self.star_id_status = self._io.read_u1()
            self.power_status = self._io.read_u4be()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Csim.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Csim.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Csim.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class BeaconLong(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.offset_0 = []
            for i in range(12):
                self.offset_0.append(self._io.read_u1())

            self.l0_status = self._io.read_u1()
            self.l0_acpt_cnt = self._io.read_u1()
            self.l0_rjct_cnt = self._io.read_u1()
            self.hw_sec_cnt = self._io.read_u1()
            self.offset_1 = []
            for i in range(8):
                self.offset_1.append(self._io.read_u1())

            self.time_tag = self._io.read_u4be()
            self.offset_2 = []
            for i in range(4):
                self.offset_2.append(self._io.read_u1())

            self.pld_tlm_ack_cnt = self._io.read_u1()
            self.pld_cmd_cnt = self._io.read_u1()
            self.pld_tlm_to_cnt = self._io.read_u1()
            self.pld_tlm_nak_cnt = self._io.read_u1()
            self.spare_end = self._io.read_u8be()
            self.cmd_status = self._io.read_u1()
            self.realtime_cmd_accept_count = self._io.read_u1()
            self.realtime_cmd_reject_count = self._io.read_u1()
            self.stored_cmd_accept_cnt = self._io.read_u1()
            self.stored_cmd_reject_cnt = self._io.read_u1()
            self.macros_status_1 = self._io.read_u2be()
            self.scrub_status_overall = self._io.read_s1()
            self.scrub_count = self._io.read_u1()
            self.image_booted = self._io.read_u1()
            self.image_auto_failover = self._io.read_u1()
            self.tai_seconds = self._io.read_f8be()
            self.time_valid = self._io.read_u1()
            self.position_wrt_eci1 = self._io.read_s4be()
            self.position_wrt_eci2 = self._io.read_s4be()
            self.position_wrt_eci3 = self._io.read_s4be()
            self.velocity_wrt_eci1 = self._io.read_s4be()
            self.velocity_wrt_eci2 = self._io.read_s4be()
            self.velocity_wrt_eci3 = self._io.read_s4be()
            self.nadir_vector_body1 = self._io.read_s2be()
            self.nadir_vector_body2 = self._io.read_s2be()
            self.nadir_vector_body3 = self._io.read_s2be()
            self.sun_vector_body1 = self._io.read_s2be()
            self.sun_vector_body2 = self._io.read_s2be()
            self.sun_vector_body3 = self._io.read_s2be()
            self.sun_position_wrt_eci1 = self._io.read_s4be()
            self.sun_position_wrt_eci2 = self._io.read_s4be()
            self.sun_position_wrt_eci3 = self._io.read_s4be()
            self.moon_position_wrt_eci1 = self._io.read_s4be()
            self.moon_position_wrt_eci2 = self._io.read_s4be()
            self.moon_position_wrt_eci3 = self._io.read_s4be()
            self.refs_valid = self._io.read_u1()
            self.esm_valid = self._io.read_u1()
            self.run_low_rate_task = self._io.read_u1()
            self.q_body_wrt_eci1 = self._io.read_s4be()
            self.q_body_wrt_eci2 = self._io.read_s4be()
            self.q_body_wrt_eci3 = self._io.read_s4be()
            self.q_body_wrt_eci4 = self._io.read_s4be()
            self.body_rate1_dps = self._io.read_s4be()
            self.body_rate2_dps = self._io.read_s4be()
            self.body_rate3_dps = self._io.read_s4be()
            self.bad_att_timer = self._io.read_u4be()
            self.bad_rate_timer = self._io.read_u4be()
            self.reinit_count = self._io.read_u4be()
            self.gnc_status_1 = self._io.read_u4be()
            self.hr_cycle_safe_mode = self._io.read_u4be()
            self.rotisserie_rate_dps = self._io.read_s2be()
            self.adcs_mode = self._io.read_u1()
            self.gnc_status_2 = self._io.read_u2be()
            self.filtered_speed_rpm1 = self._io.read_s2be()
            self.filtered_speed_rpm2 = self._io.read_s2be()
            self.filtered_speed_rpm3 = self._io.read_s2be()
            self.operating_mode1 = self._io.read_u1()
            self.operating_mode2 = self._io.read_u1()
            self.operating_mode3 = self._io.read_u1()
            self.operating_mode = self._io.read_u1()
            self.star_id_step = self._io.read_u1()
            self.att_status = self._io.read_u1()
            self.det_timeout_count = self._io.read_u2be()
            self.num_attitude_stars = self._io.read_u1()
            self.position_error1 = self._io.read_s4be()
            self.position_error2 = self._io.read_s4be()
            self.position_error3 = self._io.read_s4be()
            self.eigen_error = self._io.read_u4be()
            self.time_into_search = self._io.read_u2be()
            self.wait_timer = self._io.read_u2be()
            self.sun_point_angle_error = self._io.read_u2be()
            self.sun_point_state = self._io.read_u1()
            self.momentum_vector_body1 = self._io.read_s2be()
            self.momentum_vector_body2 = self._io.read_s2be()
            self.momentum_vector_body3 = self._io.read_s2be()
            self.total_momentum_mag = self._io.read_u2be()
            self.duty_cycle1 = self._io.read_s1()
            self.duty_cycle2 = self._io.read_s1()
            self.duty_cycle3 = self._io.read_s1()
            self.torque_rod_mode1 = self._io.read_u1()
            self.torque_rod_mode2 = self._io.read_u1()
            self.torque_rod_mode3 = self._io.read_u1()
            self.mag_source_used = self._io.read_u1()
            self.momentum_vector_valid = self._io.read_u1()
            self.sun_vector_body1_meas = self._io.read_s2be()
            self.sun_vector_body2_meas = self._io.read_s2be()
            self.sun_vector_body3_meas = self._io.read_s2be()
            self.sun_vector_status = self._io.read_u1()
            self.css_invalid_count = self._io.read_u2be()
            self.sun_sensor_used = self._io.read_u1()
            self.mag_vector_body1 = self._io.read_s2be()
            self.mag_vector_body2 = self._io.read_s2be()
            self.mag_vector_body3 = self._io.read_s2be()
            self.mag_invalid_count = self._io.read_u2be()
            self.mag_vector_valid = self._io.read_u1()
            self.mag_sensor_used = self._io.read_u1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Csim.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Grizu263a(KaitaiStruct):
    """:field teamid: header.teamid
    :field year: header.year
    :field month: header.month
    :field date: header.date
    :field hour: header.hour
    :field minute: header.minute
    :field second: header.second
    :field temp: header.temp
    :field epstoobcina1_current: header.epstoobcina1_current
    :field epstoobcina1_busvoltage: header.epstoobcina1_busvoltage
    :field epsina2_current: header.epsina2_current
    :field epsina2_busvoltage: header.epsina2_busvoltage
    :field baseina3_current: header.baseina3_current
    :field baseina3_busvoltage: header.baseina3_busvoltage
    :field topina4_current: header.topina4_current
    :field topina4_busvoltage: header.topina4_busvoltage
    :field behindantenina5_current: header.behindantenina5_current
    :field behindantenina5_busvoltage: header.behindantenina5_busvoltage
    :field rightsideina6_current: header.rightsideina6_current
    :field rightsideina6_busvoltage: header.rightsideina6_busvoltage
    :field leftsideina7_current: header.leftsideina7_current
    :field leftsideina7_busvoltage: header.leftsideina7_busvoltage
    :field imumx: header.imumx
    :field imumy: header.imumy
    :field imumz: header.imumz
    :field imuax: header.imuax
    :field imuay: header.imuay
    :field imuaz: header.imuaz
    :field imugx: header.imugx
    :field imugy: header.imugy
    :field imugz: header.imugz
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.header = Grizu263a.Header(self._io, self, self._root)

    class Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pad = self._io.read_bytes(1)
            self.teamid = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.teamid == u"YM2VRZ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.teamid, self._io, u"/types/header/seq/1")
            self.year = self._io.read_u1()
            self.month = self._io.read_u1()
            self.date = self._io.read_u1()
            self.hour = self._io.read_u1()
            self.minute = self._io.read_u1()
            self.second = self._io.read_u1()
            self.temp = self._io.read_s2le()
            self.epstoobcina1_current = self._io.read_u2le()
            self.epstoobcina1_busvoltage = self._io.read_u2le()
            self.epsina2_current = self._io.read_u2le()
            self.epsina2_busvoltage = self._io.read_u2le()
            self.baseina3_current = self._io.read_u2le()
            self.baseina3_busvoltage = self._io.read_u2le()
            self.topina4_current = self._io.read_u2le()
            self.topina4_busvoltage = self._io.read_u2le()
            self.behindantenina5_current = self._io.read_u2le()
            self.behindantenina5_busvoltage = self._io.read_u2le()
            self.rightsideina6_current = self._io.read_u2le()
            self.rightsideina6_busvoltage = self._io.read_u2le()
            self.leftsideina7_current = self._io.read_u2le()
            self.leftsideina7_busvoltage = self._io.read_u2le()
            self.imumx = self._io.read_s2le()
            self.imumy = self._io.read_s2le()
            self.imumz = self._io.read_s2le()
            self.imuax = self._io.read_s2le()
            self.imuay = self._io.read_s2le()
            self.imuaz = self._io.read_s2le()
            self.imugx = self._io.read_s2le()
            self.imugy = self._io.read_s2le()
            self.imugz = self._io.read_s2le()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Asuphoenix(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field priority: ax25_frame.payload.ax25_info.csp_header.priority
    :field source: ax25_frame.payload.ax25_info.csp_header.source
    :field destination: ax25_frame.payload.ax25_info.csp_header.destination
    :field destination_port: ax25_frame.payload.ax25_info.csp_header.destination_port
    :field source_port: ax25_frame.payload.ax25_info.csp_header.source_port
    :field reserved: ax25_frame.payload.ax25_info.csp_header.reserved
    :field hmac: ax25_frame.payload.ax25_info.csp_header.hmac
    :field xtea: ax25_frame.payload.ax25_info.csp_header.xtea
    :field rdp: ax25_frame.payload.ax25_info.csp_header.rdp
    :field crc: ax25_frame.payload.ax25_info.csp_header.crc
    :field comms_idx_int: ax25_frame.payload.ax25_info.csp_node.csp_node_port.comms_idx_int
    :field total_obc_resets: ax25_frame.payload.ax25_info.csp_node.csp_node_port.total_obc_resets
    :field current_bat_volt_flt: ax25_frame.payload.ax25_info.csp_node.csp_node_port.current_bat_volt_flt
    :field obc_clock: ax25_frame.payload.ax25_info.csp_node.csp_node_port.obc_clock
    :field current_3v3_flt: ax25_frame.payload.ax25_info.csp_node.csp_node_port.current_3v3_flt
    :field current_5v_flt: ax25_frame.payload.ax25_info.csp_node.csp_node_port.current_5v_flt
    :field current_adcs_flt: ax25_frame.payload.ax25_info.csp_node.csp_node_port.current_adcs_flt
    :field eps_charge_volt_bat_flt: ax25_frame.payload.ax25_info.csp_node.csp_node_port.eps_charge_volt_bat_flt
    :field eps_charge_current_bat: ax25_frame.payload.ax25_info.csp_node.csp_node_port.eps_charge_current_bat
    :field eps_temp: ax25_frame.payload.ax25_info.csp_node.csp_node_port.eps_temp
    :field bat_temp: ax25_frame.payload.ax25_info.csp_node.csp_node_port.bat_temp
    :field brownouts: ax25_frame.payload.ax25_info.csp_node.csp_node_port.brownouts
    :field ax100_rssi: ax25_frame.payload.ax25_info.csp_node.csp_node_port.ax100_rssi
    :field ax100_board_temp: ax25_frame.payload.ax25_info.csp_node.csp_node_port.ax100_board_temp
    :field gps_sats_used: ax25_frame.payload.ax25_info.csp_node.csp_node_port.gps_sats_used
    :field ants_deployed: ax25_frame.payload.ax25_info.csp_node.csp_node_port.ants_deployed
    :field gpio_state: ax25_frame.payload.ax25_info.csp_node.csp_node_port.gpio_state
    :field temp_brd: ax25_frame.payload.ax25_info.csp_node.csp_node_port.temp_brd
    :field temp_pa: ax25_frame.payload.ax25_info.csp_node.csp_node_port.temp_pa
    :field last_rssi: ax25_frame.payload.ax25_info.csp_node.csp_node_port.last_rssi
    :field last_rferr: ax25_frame.payload.ax25_info.csp_node.csp_node_port.last_rferr
    :field tx_count: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tx_count
    :field rx_count: ax25_frame.payload.ax25_info.csp_node.csp_node_port.rx_count
    :field tx_bytes: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tx_bytes
    :field rx_bytes: ax25_frame.payload.ax25_info.csp_node.csp_node_port.rx_bytes
    :field active_conf: ax25_frame.payload.ax25_info.csp_node.csp_node_port.active_conf
    :field boot_count: ax25_frame.payload.ax25_info.csp_node.csp_node_port.boot_count
    :field boot_cause: ax25_frame.payload.ax25_info.csp_node.csp_node_port.boot_cause
    :field last_contact: ax25_frame.payload.ax25_info.csp_node.csp_node_port.last_contact
    :field bgnd_rssi: ax25_frame.payload.ax25_info.csp_node.csp_node_port.bgnd_rssi
    :field tx_duty: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tx_duty
    :field tot_tx_count: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tot_tx_count
    :field tot_rx_count: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tot_rx_count
    :field tot_tx_bytes: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tot_tx_bytes
    :field tot_rx_bytes: ax25_frame.payload.ax25_info.csp_node.csp_node_port.tot_rx_bytes
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Asuphoenix.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Asuphoenix.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Asuphoenix.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Asuphoenix.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Asuphoenix.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Asuphoenix.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Asuphoenix.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Asuphoenix.IFrame(self._io, self, self._root)


    class ObcHkT(KaitaiStruct):
        """
        .. seealso::
           Source - http://phxcubesat.asu.edu/content/amateur-operations
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_type_magic = self._io.read_bytes(3)
            if not self.beacon_type_magic == b"\x68\x6B\x3A":
                raise kaitaistruct.ValidationNotEqualError(b"\x68\x6B\x3A", self.beacon_type_magic, self._io, u"/types/obc_hk_t/seq/0")
            self.int_comms_idx_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.total_obc_resets_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.current_bat_volt_int_str = (self._io.read_bytes_term(46, False, True, True)).decode(u"utf-8")
            self.current_bat_volt_frac_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.obc_disk_space_used_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.obc_clock_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.current_3v3_int_str = (self._io.read_bytes_term(46, False, True, True)).decode(u"utf-8")
            self.current_3v3_frac_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.current_5v_int_str = (self._io.read_bytes_term(46, False, True, True)).decode(u"utf-8")
            self.current_5v_frac_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.current_adcs_int_str = (self._io.read_bytes_term(46, False, True, True)).decode(u"utf-8")
            self.current_adcs_frac_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.eps_charge_volt_bat_int_str = (self._io.read_bytes_term(46, False, True, True)).decode(u"utf-8")
            self.eps_charge_volt_bat_frac_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.eps_charge_current_bat_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.eps_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.bat_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.brownouts_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.ax100_rssi_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.ax100_board_temp_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.gps_sats_used_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.ants_deployed_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf-8")
            self.gpio_state_str = (self._io.read_bytes(1)).decode(u"utf-8")

        @property
        def obc_clock(self):
            if hasattr(self, '_m_obc_clock'):
                return self._m_obc_clock

            self._m_obc_clock = int(self.obc_clock_str)
            return getattr(self, '_m_obc_clock', None)

        @property
        def ax100_board_temp(self):
            if hasattr(self, '_m_ax100_board_temp'):
                return self._m_ax100_board_temp

            self._m_ax100_board_temp = int(self.ax100_board_temp_str)
            return getattr(self, '_m_ax100_board_temp', None)

        @property
        def total_obc_resets(self):
            if hasattr(self, '_m_total_obc_resets'):
                return self._m_total_obc_resets

            self._m_total_obc_resets = int(self.total_obc_resets_str)
            return getattr(self, '_m_total_obc_resets', None)

        @property
        def gps_sats_used(self):
            if hasattr(self, '_m_gps_sats_used'):
                return self._m_gps_sats_used

            self._m_gps_sats_used = int(self.gps_sats_used_str)
            return getattr(self, '_m_gps_sats_used', None)

        @property
        def ax100_rssi(self):
            if hasattr(self, '_m_ax100_rssi'):
                return self._m_ax100_rssi

            self._m_ax100_rssi = int(self.ax100_rssi_str)
            return getattr(self, '_m_ax100_rssi', None)

        @property
        def current_5v_flt(self):
            if hasattr(self, '_m_current_5v_flt'):
                return self._m_current_5v_flt

            self._m_current_5v_flt = (int(self.current_5v_int_str) + ((((int(int(self.current_5v_int_str) < 0) * -2) + 1) * int(self.current_5v_frac_str)) / 1000.0))
            return getattr(self, '_m_current_5v_flt', None)

        @property
        def ants_deployed(self):
            if hasattr(self, '_m_ants_deployed'):
                return self._m_ants_deployed

            self._m_ants_deployed = int(self.ants_deployed_str)
            return getattr(self, '_m_ants_deployed', None)

        @property
        def eps_temp(self):
            if hasattr(self, '_m_eps_temp'):
                return self._m_eps_temp

            self._m_eps_temp = int(self.eps_temp_str)
            return getattr(self, '_m_eps_temp', None)

        @property
        def current_adcs_flt(self):
            if hasattr(self, '_m_current_adcs_flt'):
                return self._m_current_adcs_flt

            self._m_current_adcs_flt = (int(self.current_adcs_int_str) + ((((int(int(self.current_adcs_int_str) < 0) * -2) + 1) * int(self.current_adcs_frac_str)) / 1000.0))
            return getattr(self, '_m_current_adcs_flt', None)

        @property
        def bat_temp(self):
            if hasattr(self, '_m_bat_temp'):
                return self._m_bat_temp

            self._m_bat_temp = int(self.bat_temp_str)
            return getattr(self, '_m_bat_temp', None)

        @property
        def comms_idx_int(self):
            if hasattr(self, '_m_comms_idx_int'):
                return self._m_comms_idx_int

            self._m_comms_idx_int = int(self.int_comms_idx_str)
            return getattr(self, '_m_comms_idx_int', None)

        @property
        def current_3v3_flt(self):
            if hasattr(self, '_m_current_3v3_flt'):
                return self._m_current_3v3_flt

            self._m_current_3v3_flt = (int(self.current_3v3_int_str) + ((((int(int(self.current_3v3_int_str) < 0) * -2) + 1) * int(self.current_3v3_frac_str)) / 1000.0))
            return getattr(self, '_m_current_3v3_flt', None)

        @property
        def gpio_state(self):
            if hasattr(self, '_m_gpio_state'):
                return self._m_gpio_state

            self._m_gpio_state = int(self.gpio_state_str)
            return getattr(self, '_m_gpio_state', None)

        @property
        def current_bat_volt_flt(self):
            if hasattr(self, '_m_current_bat_volt_flt'):
                return self._m_current_bat_volt_flt

            self._m_current_bat_volt_flt = (int(self.current_bat_volt_int_str) + ((((int(int(self.current_bat_volt_int_str) < 0) * -2) + 1) * int(self.current_bat_volt_frac_str)) / 100.0))
            return getattr(self, '_m_current_bat_volt_flt', None)

        @property
        def eps_charge_current_bat(self):
            if hasattr(self, '_m_eps_charge_current_bat'):
                return self._m_eps_charge_current_bat

            self._m_eps_charge_current_bat = int(self.eps_charge_current_bat_str)
            return getattr(self, '_m_eps_charge_current_bat', None)

        @property
        def brownouts(self):
            if hasattr(self, '_m_brownouts'):
                return self._m_brownouts

            self._m_brownouts = int(self.brownouts_str)
            return getattr(self, '_m_brownouts', None)

        @property
        def eps_charge_volt_bat_flt(self):
            if hasattr(self, '_m_eps_charge_volt_bat_flt'):
                return self._m_eps_charge_volt_bat_flt

            self._m_eps_charge_volt_bat_flt = (int(self.eps_charge_volt_bat_int_str) + ((((int(int(self.eps_charge_volt_bat_int_str) < 0) * -2) + 1) * int(self.eps_charge_volt_bat_frac_str)) / 100.0))
            return getattr(self, '_m_eps_charge_volt_bat_flt', None)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Asuphoenix.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Asuphoenix.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Asuphoenix.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Asuphoenix.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Asuphoenix.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Ax100ControlPortT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.temp_brd = self._io.read_s2be()
            self.temp_pa = self._io.read_s2be()
            self.last_rssi = self._io.read_s2be()
            self.last_rferr = self._io.read_s2be()
            self.tx_count = self._io.read_u4be()
            self.rx_count = self._io.read_u4be()
            self.tx_bytes = self._io.read_u4be()
            self.rx_bytes = self._io.read_u4be()
            self.active_conf = self._io.read_u1()
            self.boot_count = self._io.read_u2be()
            self.boot_cause = self._io.read_u4be()
            self.last_contact = self._io.read_u4be()
            self.bgnd_rssi = self._io.read_s2be()
            self.tx_duty = self._io.read_u1()
            self.tot_tx_count = self._io.read_u4be()
            self.tot_rx_count = self._io.read_u4be()
            self.tot_tx_bytes = self._io.read_u4be()
            self.tot_rx_bytes = self._io.read_u4be()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Asuphoenix.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Asuphoenix.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Asuphoenix.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Asuphoenix.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class CspHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.raw_csp_header = self._io.read_u4be()

        @property
        def source(self):
            if hasattr(self, '_m_source'):
                return self._m_source

            self._m_source = ((self.raw_csp_header >> 25) & 31)
            return getattr(self, '_m_source', None)

        @property
        def source_port(self):
            if hasattr(self, '_m_source_port'):
                return self._m_source_port

            self._m_source_port = ((self.raw_csp_header >> 8) & 63)
            return getattr(self, '_m_source_port', None)

        @property
        def destination_port(self):
            if hasattr(self, '_m_destination_port'):
                return self._m_destination_port

            self._m_destination_port = ((self.raw_csp_header >> 14) & 63)
            return getattr(self, '_m_destination_port', None)

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp

            self._m_rdp = ((self.raw_csp_header & 2) >> 1)
            return getattr(self, '_m_rdp', None)

        @property
        def destination(self):
            if hasattr(self, '_m_destination'):
                return self._m_destination

            self._m_destination = ((self.raw_csp_header >> 20) & 31)
            return getattr(self, '_m_destination', None)

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority

            self._m_priority = (self.raw_csp_header >> 30)
            return getattr(self, '_m_priority', None)

        @property
        def reserved(self):
            if hasattr(self, '_m_reserved'):
                return self._m_reserved

            self._m_reserved = ((self.raw_csp_header >> 4) & 15)
            return getattr(self, '_m_reserved', None)

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea

            self._m_xtea = ((self.raw_csp_header & 4) >> 2)
            return getattr(self, '_m_xtea', None)

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac

            self._m_hmac = ((self.raw_csp_header & 8) >> 3)
            return getattr(self, '_m_hmac', None)

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc

            self._m_crc = (self.raw_csp_header & 1)
            return getattr(self, '_m_crc', None)


    class Ax100T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.csp_header.source_port
            if _on == 0:
                self.csp_node_port = Asuphoenix.Ax100ControlPortT(self._io, self, self._root)


    class ObcT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.csp_header.source_port
            if _on == 27:
                self.csp_node_port = Asuphoenix.ObcHkT(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (0), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Asuphoenix.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_header = Asuphoenix.CspHeaderT(self._io, self, self._root)
            _on = self.csp_header.source
            if _on == 2:
                self.csp_node = Asuphoenix.ObcT(self._io, self, self._root)
            elif _on == 5:
                self.csp_node = Asuphoenix.Ax100T(self._io, self, self._root)



